@@ -4,7 +4,7 @@
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;; Author:    David Aspinall <David.Aspinall@ed.ac.uk>
 ;;
-;; acl2.el,v 8.0 2004/04/17 23:39:58 da Exp
+;; acl2.el,v 9.0 2008/01/30 15:22:06 da Exp
 ;;
 ;; Needs improvement!
 ;;
@@ -24,8 +24,7 @@
  proof-prog-name		 "acl2"
 
  proof-script-sexp-commands	 t
- proof-script-comment-start             ";"
- proof-script-comment-start             ";"
+ proof-script-comment-start	";"
 
  proof-shell-annotated-prompt-regexp "ACL2[ !]*>+"
 
@@ -85,9 +84,6 @@
  ;; End of easy config.
  )
 
-(eval-when-compile 
-  (require 'proof-shell))		; nuke warning
-
 ;; Interrupts and errors enter another loop; break out of it
 (add-hook
  'proof-shell-handle-error-or-interrupt-hook
@@ -97,6 +93,6 @@
 
 
 (warn "ACL2 Proof General is incomplete!  Please help improve it!
-Read the manual, make improvements and send them to da+pg-feedback@inf.ed.ac.uk")
+Please add improvements at http://proofgeneral.inf.ed.ac.uk/trac")
 
 (provide 'acl2)
