@@ -7,11 +7,14 @@
 ;; Directly inspired from x-symbol-isabelle.el of ProofGeneral by
 ;; Markus Wenzel, Christoph Wedler, David Aspinall.
 ;; 
-;; x-symbol-coq.el,v 8.0 2004/04/17 23:39:58 da Exp
+;; x-symbol-coq.el,v 9.0 2008/01/30 15:22:07 da Exp
 ;;
 ;; NB: This file is part of the Proof General distribution.
 ;;
 
+(eval-when-compile
+  (require 'proof-utils)		; to properly compile 'proof-ass'
+  (require 'cl))			; to properly compile 'block'
 
 (defvar x-symbol-coq-required-fonts nil)
   
@@ -52,7 +55,8 @@ See `x-symbol-header-groups-alist'."
    :decode-spec nil
    ;; decode-spec seems to go with highlighting encoding??
    :decode-regexp "\\([a-zA-Z0-9]+\\)\\|\\([]><=\\/~&+-*%!{}:-]+\\)"
-   :token-list #'x-symbol-coq-default-token-list))
+   :token-list #'x-symbol-coq-default-token-list
+	:input-spec nil))
 
 ;(defvar x-symbol-coq-input-token-grammar
 ;  '("\\([_'a-zA-Z0-9]+\\)\\|\\([]><=\\/~&+-*%!{}:-]+\\)"
@@ -93,15 +97,14 @@ See `x-symbol-header-groups-alist'."
 ;;;===========================================================================
 ;;  super- and subscripts
 ;;;===========================================================================
-;not implemeted yet
 
 (defcustom x-symbol-coq-subscript-matcher 'x-symbol-coq-subscript-matcher
-  "Function matching super-/subscripts for language `isa'.
+  "Function matching super-/subscripts for language `coq'.
 See language access `x-symbol-LANG-subscript-matcher'."
   :group 'x-symbol-coq
   :type 'function)
 
-(defcustom x-symbol-coq-font-lock-regexp "_{\\|__\\|\\^{\\|\\^\\^"
+(defcustom x-symbol-coq-font-lock-regexp ",{\\|__\\|\\^{\\|\\^\\^"
   "Regexp matching the start tag of Coq super- and subscripts."
   :group 'x-symbol-coq
   :type 'regexp)
@@ -130,7 +133,8 @@ or subscript tag."
       (while (re-search-forward x-symbol-coq-font-lock-regexp limit t)
         (setq open-beg (match-beginning 0)
               open-end (match-end 0)
-              script-type (if (eq (char-after (- open-end 2)) ?_)
+              script-type (if (or (eq (char-after (- open-end 2)) ?_)
+											 (eq (char-after (- open-end 2)) ?,))
                               'x-symbol-sub-face
 									 'x-symbol-sup-face))
         (if (or (eq (char-after (- open-end 1)) ?{)) ; if is spanning sup-/subscript
@@ -159,11 +163,11 @@ or subscript tag."
 			 )))))
    
 (defun coq-match-subscript (limit)
-  (if (proof-ass x-symbol-enable)
+  (if coq-x-symbol-enable
       (setq x-symbol-coq-subscript-type
             (funcall x-symbol-coq-subscript-matcher limit))))
 
-;(defvar x-symbol-coq-font-lock-keywords nil)
+
 
 (defvar x-symbol-coq-font-lock-allowed-faces t)
 ;;;===========================================================================
@@ -241,7 +245,7 @@ See `x-symbol-language-access-alist' for details."
     (logicaland "/\\")
     (logicalor "\\/")
     (universal1 "forall")
-;    (existential1 "ex")
+    (existential1 "exists")
     (biglogicaland "And")
     (ceilingleft "lceil")
     (ceilingright "rceil")
@@ -472,7 +476,7 @@ and `x-symbol-coq-xsymbol-table'."
     x-symbol-coq-xsymbol-table)))
 
 (defcustom x-symbol-coq-auto-style
-  '((proof-ass x-symbol-enable)	 ; MODE-ON: whether to turn on interactively
+  '(coq-x-symbol-enable	          ; MODE-ON: whether to turn on interactively
     nil   ;; x-symbol-coding
     'null ;; x-symbol-8bits	   [NEVER want it; null disables search]
     nil   ;; x-symbol-unique
