@@ -6,13 +6,17 @@
 ;; License:      GPL (GNU GENERAL PUBLIC LICENSE)
 ;; Status:       Experimental code
 ;;
-;; proof-depends.el,v 8.0 2004/04/17 23:39:59 da Exp
+;; proof-depends.el,v 9.0 2008/01/30 15:22:16 da Exp
 ;; 
 ;; Based on Fiona McNeill's MSc project on analysing dependencies
 ;; within proofs.  Code rewritten by David Aspinall.
 ;; 
 
+(require 'cl)
 (require 'span)
+(require 'pg-vars)
+(require 'proof-config)
+(require 'proof-autoloads)
 
 ;; Variables
 
@@ -71,9 +75,9 @@ This is done using `proof-depends-module-name-for-buffer' and
 (defun proof-depends-process-dependencies (name gspan)
   "Process dependencies reported by prover, for NAME in span GSPAN.
 Called from `proof-done-advancing' when a save is processed and
-proof-last-theorem-dependencies is set."
+`proof-last-theorem-dependencies' is set."
 
-  (set-span-property gspan 'dependencies
+  (span-set-property gspan 'dependencies
 		     ;; Ancestors of NAME are in the second component.
 		     ;; FIXME: for now we ignore the first component:
 		     ;; NAME may not be enough [Isar allows proof regions
@@ -89,7 +93,7 @@ proof-last-theorem-dependencies is set."
 	 ;; and return resulting list paired up with names.
 	 (depspans    
 	  (apply 'append
-		 (mapcar-spans 
+		 (span-mapcar-spans 
 		  (lambda (depspan)
 		    (let ((dname (span-property depspan 'name)))
 		      (if (and
@@ -97,7 +101,7 @@ proof-last-theorem-dependencies is set."
 			   (member dname samefilenames))
 			  (let ((forwarddeps 
 				 (span-property depspan 'dependents)))
-			    (set-span-property depspan 'dependents
+			    (span-set-property depspan 'dependents
 					       (cons
 						(list name gspan) forwarddeps))
 			    ;; return list of args for menu fun: name and span
@@ -106,7 +110,7 @@ proof-last-theorem-dependencies is set."
 		  (span-start gspan)
 		  'type))))
     
-    (set-span-property gspan 'dependencies-within-file depspans)
+    (span-set-property gspan 'dependencies-within-file depspans)
     (setq proof-last-theorem-dependencies nil)))
 
 
@@ -146,7 +150,7 @@ proof-last-theorem-dependencies is set."
 
 (defun proof-dep-alldeps-menu (span)
   (or (span-property span 'dependencies-menu) ;; cached value
-      (set-span-property span 'dependencies-menu
+      (span-set-property span 'dependencies-menu
 			 (proof-dep-make-alldeps-menu 
 			  (span-property span 'dependencies)))))
 
@@ -229,21 +233,21 @@ This is simply to display the dependency somehow."
   (let ((helpmsg  (concat "This item is a dependency (ancestor) of " name)))
     (while nmspans
       (let ((span (cadar nmspans)))
-	(set-span-property span 'face 'proof-highlight-dependency-face)
-	(set-span-property span 'priority pg-dep-span-priority)
-	(set-span-property span 'mouse-highlight nil)
-	(set-span-property span 'help-echo helpmsg))
+	(span-set-property span 'face 'proof-highlight-dependency-face)
+	(span-set-property span 'priority pg-dep-span-priority)
+	(span-set-property span 'mouse-highlight nil)
+	(span-set-property span 'help-echo helpmsg))
       (setq nmspans (cdr nmspans)))))
 
 (defun proof-highlight-depts (name nmspans)
   (let ((helpmsg  (concat "This item depends on (is a child of) " name)))
     (while nmspans
       (let ((span (cadar nmspans)))
-	(set-span-property span 'face 'proof-highlight-dependent-face)
-	(set-span-property span 'priority pg-dep-span-priority)
-	(set-span-property span 'mouse-highlight nil)
-	(set-span-property span 'help-echo helpmsg)
-	(set-span-property span 'balloon-help helpmsg))
+	(span-set-property span 'face 'proof-highlight-dependent-face)
+	(span-set-property span 'priority pg-dep-span-priority)
+	(span-set-property span 'mouse-highlight nil)
+	(span-set-property span 'help-echo helpmsg)
+	(span-set-property span 'balloon-help helpmsg))
       (setq nmspans (cdr nmspans)))))
 
 (defun proof-dep-unhighlight ()
@@ -255,8 +259,8 @@ This is simply to display the dependency somehow."
     (let ((span (span-at (point-min) 'type)))
       (while span
 	(pg-set-span-helphighlights span 'nohighlight)
-	(set-span-property span 'face 'proof-locked-face)
-	(set-span-property span 'priority pg-ordinary-span-priority)
+	(span-set-property span 'face 'proof-locked-face)
+	(span-set-property span 'priority pg-ordinary-span-priority)
 	(setq span (next-span span 'type))))))
 
 
