@@ -4,13 +4,14 @@
 ;; Author:    David Aspinall <David.Aspinall@ed.ac.uk>
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-easy-config.el,v 8.0 2004/04/17 23:39:59 da Exp
+;; proof-easy-config.el,v 9.0 2008/01/30 15:22:16 da Exp
 ;;
-;; Future version might copy settings instead; consider how best to
+;; Future versions might copy settings instead; consider how best to
 ;; interface with customization mechanism so a new prover can be
 ;; configured by editing inside custom buffers.
 ;;
-(require 'proof)
+
+(require 'proof-site)			; proof-assistant, proof-assistant-symbol
 
 (defconst proof-easy-config-derived-modes-table
   '((""         "script"     proof-mode (proof-config-done))
@@ -30,7 +31,6 @@
 	   (hyphen    (if (string-equal prefixsym "") "" "-"))
 	   (mode      (intern (concat modert hyphen "mode")))
 	   (modename  (concat proof-assistant " " suffixnm))
-	   (varname   (intern (concat "proof-mode-for-" suffixnm)))
 	   ;; FIXME: declare these variables in proof-config:
 	   ;;	proof-{goals,resp,trace}-font-lock-keywords,
 	   ;;   proof-{goals,resp,trace}-syntax-table-entries
@@ -51,10 +51,7 @@
 				      (setq syn (cddr syn))))))
 		       body)))
       (eval
-       `(define-derived-mode ,mode ,parent ,modename nil ,@fullbody))
-      ;; Set proof-mode-for-script and friends
-      ;; NB: top-level, so we don't need proof-pre-shell-start-hook.
-      (set varname mode))))
+       `(define-derived-mode ,mode ,parent ,modename nil ,@fullbody)))))
 
 (defun proof-easy-config-check-setup (sym name)
   "A number of simple checks."
@@ -63,11 +60,11 @@
     ;; in the macro matches that in `proof-assistant-table'
     ;; and have the right type.
     (unless (symbolp sym)
-      (error "proof-easy-config: first argument (%s) should be a symbol"
+      (error "Macro proof-easy-config: first argument (%s) should be a symbol"
 	     sym))
     (unless (stringp name)
-      (error "proof-easy-config: second argument (%s) should be a string"
-	     string))
+      (error "Macro proof-easy-config: second argument (%s) should be a string"
+	     name))
     (cond
      ((or 
        (and (boundp 'proof-assistant) proof-assistant 
