@@ -3,12 +3,13 @@
 ;; Authors:	   Markus Wenzel, David Aspinall
 ;; License:        GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-indent.el,v 8.0 2004/04/17 23:39:59 da Exp
+;; proof-indent.el,v 9.0 2008/01/30 15:22:16 da Exp
 ;;
 
-(require 'proof)                        ; loader
-(require 'proof-script)                 ; indent code is for script editing
-
+(require 'proof-config)			; config variables
+(require 'proof-utils)			; proof-ass
+(require 'proof-syntax)			; p-looking-at-safe, p-re-search
+(require 'proof-autoloads)		; p-locked-end
 
 (defun proof-indent-indent ()
   "Determine indentation caused by syntax element at current point."
@@ -59,7 +60,8 @@
        (found-prev (proof-indent-goto-prev)))
     (if (not found-prev) (goto-char current))   ; recover position
     (cond
-     ((and found-prev (or proof-indent-hang (= (current-indentation) (current-column))))
+     ((and found-prev (or proof-indent-hang 
+			  (= (current-indentation) (current-column))))
       (+ indent
          (current-column)
          (if (and inner (not (proof-indent-inner-p))) 0 (proof-indent-indent))
@@ -82,47 +84,9 @@
               (indent-line-to
                (max 0 (save-excursion
                         (back-to-indentation)
-                        (proof-indent-calculate (proof-indent-offset) (proof-indent-inner-p))))))
+                        (proof-indent-calculate 
+			 (proof-indent-offset) (proof-indent-inner-p))))))
             (if (< (current-column) (current-indentation))
-                (back-to-indentation))))
-  (if proof-indent-pad-eol
-      (proof-indent-pad-eol)))
+                (back-to-indentation)))))
       
-
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;
-;; Useless spaces
-;;
-
-(defun proof-indent-pad-eol (&optional col)
-  "Add padding to end of current line to match the previous line or COL.
-This function adds useless space to buffers, but it cleans up the appearance
-of locked regions in XEmacs."
-  ;; A hook for TAB?
-  (interactive)
-  (save-excursion
-    (let ((descol (or col
-		     (save-excursion
-		       (end-of-line 0)
-		       (current-column)))))
-      (save-excursion
-	(end-of-line 1)
-	(if (or (equal (current-column) 0) ;; special case for empty lines
-		(> descol (current-column)))
-	    (progn
-	      (insert-char ?\   (- descol (current-column)))))))))
-
-(defun proof-indent-pad-eol-region (start end)
-  "Pad a region with extra spaces to the length of the longest line."
-  (interactive)
-  ;; Find the longest line..
-  ;; pad others to the same length
-  )
-	  
-
-
-
-
-
 (provide 'proof-indent)
