@@ -4,12 +4,10 @@
 ;; Authors:   David Aspinall
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-menu.el,v 8.0 2004/04/17 23:39:59 da Exp
+;; proof-menu.el,v 9.1 2008/01/31 12:34:17 da Exp
 ;;
 
-(require 'proof)
-(require 'proof-syntax)
-(require 'proof-toolbar)     ; needed for proof-toolbar-scripting-menu
+(require 'proof)           ; proof-deftoggle, proof-eval-when-ready-for-assistant
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -78,55 +76,57 @@ without adjusting window layout."
 
 ;;;###autoload
 (defun proof-menu-define-keys (map)
-;; M-a and M-e are usually {forward,backward}-sentence.
-;; Some modes also override these with similar commands
-(define-key map [(meta a)] 'proof-backward-command)
-(define-key map [(meta e)] 'proof-forward-command)
-(define-key map [(meta up)] 'proof-backward-command)
-(define-key map [(meta down)] 'proof-forward-command)
-(define-key map [(control meta a)] 'proof-goto-command-start)
-(define-key map [(control meta e)] 'proof-goto-command-end)
-(define-key map [(control c) (control a)] (proof-ass keymap))
-(define-key map [(control c) (control b)] 'proof-process-buffer)
-;; C-c C-c is proof-interrupt-process in universal-keys
-(define-key map [(control c) (control f)] 'proof-find-theorems)
-(define-key map [(control c) (control h)] 'proof-help)
-(define-key map [(control c) (control o)] 'proof-display-some-buffers)
-(define-key map [(control c) (control l)] 'proof-layout-windows)
-(define-key map [(control c) (control n)] 'proof-assert-next-command-interactive)
-(define-key map [(control c) (control p)] 'proof-prf)
-(define-key map [(control c) (control r)] 'proof-retract-buffer)
-(define-key map [(control c) (control s)] 'proof-toggle-active-scripting)
-(define-key map [(control c) (control t)] 'proof-ctxt)
-(define-key map [(control c) (control u)] 'proof-undo-last-successful-command)
-;; C-c C-w is pg-response-clear-displays in universal-keys
-(define-key map [(control c) (control z)] 'proof-frob-locked-end)
-(define-key map [(control c) (control backspace)] 'proof-undo-and-delete-last-successful-command)
-; C-c C-v is proof-minibuffer-cmd in universal-keys
-(define-key map [(control c) (control ?.)] 'proof-goto-end-of-locked)
-(define-key map [(control c) (control return)] 'proof-goto-point)
-(define-key map [(control c) v] 'pg-toggle-visibility);; FIXME: FSF??
-(cond ((string-match "XEmacs" emacs-version)
-(define-key map [(control button3)]	  'proof-mouse-goto-point)
-(define-key map [(control button1)]	  'proof-mouse-track-insert)) ; no FSF
-      (t 
-(define-key map [(control mouse-3)]	  'proof-mouse-goto-point)))
- ; FSF
-;; NB: next binding overwrites comint-find-source-code.  
-;; FIXME: not implemented yet 
-;; (define-key map [(meta p)]		  'proof-previous-matching-command)
-;; (define-key map [(meta n)]		  'proof-next-matching-command)
-;; Standard binding for completion
-(define-key map [(control return)] 'proof-script-complete)
-(define-key map [(control c) (control ?\;)] 'pg-insert-last-output-as-comment)
-;;
-;; Experimental: span moving functions
-(if proof-experimental-features (progn
-      (define-key map [(control meta up)] 'pg-move-region-up)
-      (define-key map [(control meta down)] 'pg-move-region-down)))
-;; Add the universal keys bound in all PG buffers.
-;; C-c ` is next-error in universal-keys
-(proof-define-keys map proof-universal-keys))
+  ;; Prover specific keymap under C-c C-a 
+  (proof-eval-when-ready-for-assistant
+      (define-key map [(control c) (control a)] (proof-ass keymap)))
+  ;; M-a and M-e are usually {forward,backward}-sentence.
+  ;; Some modes also override these with similar commands
+  (define-key map [(meta a)] 'proof-backward-command)
+  (define-key map [(meta e)] 'proof-forward-command)
+  (define-key map [(meta up)] 'proof-backward-command)
+  (define-key map [(meta down)] 'proof-forward-command)
+  (define-key map [(control meta a)] 'proof-goto-command-start)
+  (define-key map [(control meta e)] 'proof-goto-command-end)
+  (define-key map [(control c) (control b)] 'proof-process-buffer)
+  ;; C-c C-c is proof-interrupt-process in universal-keys
+  ;; C-c C-f is proof-find-theorems in universal-keys
+  (define-key map [(control c) (control h)] 'proof-help)
+  ;; C-c C-l is proof-layout-windows in universal-keys
+  ;; C-c C-n is proof-assert-next-command-interactive in universal-keys
+  ;; C-c C-o is proof-display-some-buffers in universal-keys
+  (define-key map [(control c) (control p)] 'proof-prf)
+  (define-key map [(control c) (control r)] 'proof-retract-buffer)
+  (define-key map [(control c) (control s)] 'proof-toggle-active-scripting)
+  (define-key map [(control c) (control t)] 'proof-ctxt)
+  ;; C-c C-u is proof-undo-last-successful-command in universal-keys
+  ;; C-c C-w is pg-response-clear-displays in universal-keys
+  (define-key map [(control c) (control z)] 'proof-frob-locked-end)
+  (define-key map [(control c) (control backspace)] 
+    'proof-undo-and-delete-last-successful-command)
+  ;; C-c C-v is proof-minibuffer-cmd in universal-keys
+  ;; C-c C-. is proof-goto-end-of-locked in universal-keys
+  (define-key map [(control c) (control return)] 'proof-goto-point)
+  (define-key map [(control c) v] 'pg-toggle-visibility);; FIXME: Emacs binding?
+  (cond ((featurep 'xemacs)
+	 (define-key map [(control button3)] 'proof-mouse-goto-point)
+	 (define-key map [(control button1)] 'proof-mouse-track-insert)) ; no Emacs binding
+	(t 
+	 (define-key map [(control mouse-3)] 'proof-mouse-goto-point)))
+  ;; NB: next binding overwrites comint-find-source-code.  
+  (define-key map [(meta p)] 'pg-previous-matching-input-from-input)
+  (define-key map [(meta n)] 'pg-next-matching-input-from-input)
+  ;; Standard binding for completion
+  (define-key map [(control return)] 'proof-script-complete)
+  (define-key map [(control c) (control ?\;)] 'pg-insert-last-output-as-comment)
+  ;;
+  ;; Experimental: span moving functions
+  (if proof-experimental-features 
+      (progn
+	(define-key map [(control meta up)] 'pg-move-region-up)
+	(define-key map [(control meta down)] 'pg-move-region-down)))
+  ;; Add the universal keys bound in all PG buffers.
+  ;; NB: C-c ` is next-error in universal-keys
+  (proof-define-keys map proof-universal-keys))
 
 
 
@@ -143,10 +143,13 @@ without adjusting window layout."
    proof-mode-menu  
    proof-mode-map
    "The main Proof General menu"
-   proof-main-menu))
+   (proof-main-menu)))
 
 ;; The proof assistant specific menu
 
+(defvar proof-menu-favourites nil
+  "The Proof General favourites menu for the current proof assistant.")
+
 ;;;###autoload
 (defun proof-menu-define-specific ()
   (easy-menu-define 
@@ -154,42 +157,47 @@ without adjusting window layout."
     proof-mode-map
     (concat "The menu for " proof-assistant)
     (cons proof-assistant
-	    (append
-	     (proof-ass menu-entries)
-	     '("----")
-	     (or proof-menu-favourites
-		 (proof-menu-define-favourites-menu))
-	     (or proof-menu-settings
-		 (proof-menu-define-settings-menu))
-	     '("----")
-	    (list
-	     (vector
-	      (concat "Start " proof-assistant)
-	      'proof-shell-start
-	      ':active '(not (proof-shell-live-buffer)))
+	  (append
+	   (proof-ass menu-entries)
+	   '("----")
+	   (or proof-menu-favourites
+	       (proof-menu-define-favourites-menu))
+	   (or proof-menu-settings
+	       (proof-menu-define-settings-menu))
+	   '("----")
+	   (list
+	    (vector
+	     (concat "Start " proof-assistant)
+	     'proof-shell-start
+	     ':active '(not (proof-shell-live-buffer)))
 	    (vector
 	     (concat "Exit " proof-assistant)
 	     'proof-shell-exit
 	     ':active '(proof-shell-live-buffer)))
-	    '("----")
-	    (list
-	     (cons "Help"
-		   (append
-		    `([,(concat proof-assistant " information")
-		       '(proof-help)
-		       ,menuvisiblep proof-info-command]
-		      [,(concat proof-assistant " web page")
-		       '(browse-url proof-assistant-home-page)
-		       ,menuvisiblep proof-assistant-home-page])
-		    (proof-ass help-menu-entries))))))))
+	   '("----")
+	   (list
+	    (cons "Help"
+		  (append
+		   (list
+		    (vector
+		     (concat proof-assistant " information")
+		     'proof-help
+		     menuvisiblep proof-info-command)
+		    (vector
+		     (concat proof-assistant " web page")
+		     '(browse-url proof-assistant-home-page)
+		     menuvisiblep proof-assistant-home-page))
+		   (proof-ass help-menu-entries))))))))
 
 (defun proof-assistant-menu-update ()
   "Update proof assistant menu in scripting buffers."
   (proof-map-buffers (proof-buffers-in-mode proof-mode-for-script)
-     (proof-menu-define-settings-menu)
-     (proof-menu-define-specific)
-     (easy-menu-remove proof-assistant-menu)
-     (easy-menu-add proof-assistant-menu proof-mode-map)))
+    ;; NB: easy-menu-remove is odd in XEmacs, it considerably changes the mode popup menu.  
+    ;; In GNU Emacs this first instruction does nothing.
+    (easy-menu-remove proof-assistant-menu)
+    (proof-menu-define-settings-menu)
+    (proof-menu-define-specific)
+    (easy-menu-add proof-assistant-menu (proof-ass mode-map))))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -199,9 +207,9 @@ without adjusting window layout."
 
 (defvar proof-help-menu
   '("Help"
+    ["About PG"        proof-splash-display-screen t]
     ["PG Info"      	(info "ProofGeneral") t]
     ["PG Homepage"  	(browse-url proof-general-home-page) t]
-    ["About PG"        proof-splash-display-screen t]
     ["Send Bug Report" proof-submit-bug-report t])
   "Proof General help menu.")
 
@@ -259,13 +267,27 @@ without adjusting window layout."
 (proof-deftoggle proof-shrink-windows-tofit)
 (proof-deftoggle proof-multiple-frames-enable proof-multiple-frames-toggle)
 (proof-deftoggle proof-three-window-enable proof-three-window-toggle)
-;; (proof-deftoggle proof-output-fontify-enable proof-output-fontify-toggle)
 (proof-deftoggle proof-disappearing-proofs)
 (proof-deftoggle proof-strict-read-only)
 
 (proof-deftoggle-fn 'proof-imenu-enable 'proof-imenu-toggle)
-(proof-deftoggle-fn (proof-ass-sym x-symbol-enable) 'proof-x-symbol-toggle)
-(proof-deftoggle-fn (proof-ass-sym mmm-enable) 'proof-mmm-toggle)
+(proof-deftoggle proof-keep-response-history)
+
+(proof-eval-when-ready-for-assistant
+  (proof-deftoggle-fn 
+   (proof-ass-sym x-symbol-enable) 'proof-x-symbol-toggle)
+  (proof-deftoggle-fn 
+   (proof-ass-sym unicode-tokens-enable) 'proof-unicode-tokens-toggle)
+  (proof-deftoggle-fn 
+   (proof-ass-sym maths-menu-enable) 'proof-maths-menu-toggle)
+  (proof-deftoggle-fn (proof-ass-sym mmm-enable) 'proof-mmm-toggle))
+
+
+(defun proof-keep-response-history ()
+  "Enable associated buffer histories following `proof-keep-response-history'."
+  (if proof-keep-response-history
+      (proof-map-buffers (proof-associated-buffers) (bufhist-init))
+    (proof-map-buffers (proof-associated-buffers) (bufhist-exit))))
 
 ;; Here is the menu
 
@@ -283,21 +305,38 @@ without adjusting window layout."
       :style toggle
       :selected proof-disappearing-proofs]
 
-;     ["Output Highlighting" proof-output-fontify-toggle
-;      :active t
-;      :style toggle
-;      :selected proof-output-fontify-enable]
-
      ["Strict Read Only" proof-strict-read-only-toggle
       :style toggle
       :selected proof-strict-read-only]
-
-     ;; X-Symbol and MM are minor modes which PG settings
-     ;; enable by default for PG buffers
-     ["X-Symbol"  (proof-x-symbol-toggle (if x-symbol-mode 0 1))
-      :active (proof-x-symbol-support-maybe-available)
+     
+     ["X-Symbol"
+      (progn
+	(unless x-symbol-mode (proof-x-symbol-toggle 0))
+	(proof-x-symbol-toggle (if x-symbol-mode 0 1)))
+      :active (and 
+	       ;; X-Symbol breaks abruptly on recent 23 versions
+	       (not (>= emacs-major-version 23))
+	       (not (and (boundp 'unicode-tokens-mode) 
+			 unicode-tokens-mode))
+	       (proof-x-symbol-support-maybe-available))
       :style toggle
       :selected (and (boundp 'x-symbol-mode) x-symbol-mode)]
+
+     ["Unicode Tokens" 
+      (progn
+	(unless unicode-tokens-mode (proof-x-symbol-toggle 0))
+	(proof-unicode-tokens-toggle (if unicode-tokens-mode 0 1)))
+      :active (and (not (and (boundp 'x-symbol-mode) x-symbol-mode))
+		   (proof-unicode-tokens-support-available))
+      :style toggle
+      :selected (and (boundp 'unicode-tokens-mode) 
+		     unicode-tokens-mode)]
+
+     ["Unicode Maths Menu" (proof-maths-menu-toggle (if maths-menu-mode 0 1))
+      :active (proof-maths-menu-support-available)
+      :style toggle
+      :selected (and (boundp 'maths-menu-mode) maths-menu-mode)]
+ 
      ["Multiple Modes" (proof-mmm-toggle (if mmm-mode 0 1))
       :active (proof-mmm-support-available)
       :style toggle
@@ -314,10 +353,23 @@ without adjusting window layout."
 			 (eq proof-buffer-type 'script))
       :style toggle
       :selected proof-toolbar-enable]
+
+;;; TODO: Add this in PG 3.7.1 once; see trac #169 
+;;;      ["Response history" proof-keep-response-history-toggle
+;;;       :style toggle
+;;;       :selected proof-keep-response-history]
+
      ["Index Menu" proof-imenu-toggle
       :active (stringp (locate-library "imenu"))
       :style toggle
       :selected proof-imenu-enable]
+
+     ;; NB: convenience; speedbar isn't saved/resumed automatically.
+     ["Speedbar" speedbar
+      :active (stringp (locate-library "speedbar"))
+      :style toggle
+      :selected (and (boundp 'speedbar-frame) speedbar-frame)]
+
      ("Display"
       ["Layout Windows" proof-layout-windows]
       ["Use Three Panes" proof-three-window-toggle
@@ -328,7 +380,7 @@ without adjusting window layout."
       ;; non-Emacs users.  Cf. Gnome usability studies: menus saying
       ;; "Web Browser" more useful to novices than menus saying "Mozilla"!!
       ["Multiple Windows" proof-multiple-frames-toggle
-       :active (display-graphic-p)
+       :active (and window-system t)
        :style toggle
        :selected proof-multiple-frames-enable]
       ["Delete Empty Panes" proof-delete-empty-windows-toggle
@@ -344,6 +396,11 @@ without adjusting window layout."
        (customize-set-variable 'proof-follow-mode 'locked)
        :style radio
        :selected (eq proof-follow-mode 'locked)]
+;; Not implemented.  See Trac #187
+;;       ["Follow On Success" 
+;;        (customize-set-variable 'proof-follow-mode 'followsuccess)
+;;        :style radio
+;;        :selected (eq proof-follow-mode 'followdown)]
       ["Follow Locked Region Down" 
        (customize-set-variable 'proof-follow-mode 'followdown)
        :style radio
@@ -386,8 +443,10 @@ without adjusting window layout."
    ;;'proof-output-fontify-enable
    'proof-strict-read-only
    (proof-ass-sym x-symbol-enable)
+   (proof-ass-sym maths-menu-enable)
    (proof-ass-sym mmm-enable)
    'proof-toolbar-enable
+   'proof-keep-response-history
    'proof-imenu-enable
    ;; Display sub-menu
    'proof-three-window-enable
@@ -442,17 +501,11 @@ without adjusting window layout."
 (defconst proof-advanced-menu
   (cons "Advanced..."
 	(append
-	 (if proof-running-on-XEmacs	;; speedbar not on standard menus
-	     '(["Speedbar" speedbar
-		:active (stringp (locate-library "speedbar"))]))
 	 '(["Complete Identifier" proof-script-complete t]
 	   ["Insert last output" pg-insert-last-output-as-comment proof-shell-last-output])
 	 (list "-----")
 	 proof-show-hide-menu
 	 (list "-----")
-	 ;; NB: customize-menu-create is buggy in some versions of GNU Emacs
-	 ;; (bad in 21.1.0, good in 21.1.1, bad in 21.2.1, ...).  Comment
-	 ;; these next lines out if you must use one of these versions.
 	 (list (customize-menu-create 'proof-general))
 	 (list (customize-menu-create 'proof-general-internals "Internals"))))
   "Advanced sub-menu of script functions and customize.")
@@ -467,23 +520,24 @@ without adjusting window layout."
   "The Proof General generic menu for scripting buffers.")
 
 
-(defvar proof-main-menu
+(defun proof-main-menu ()
+  "Construct and return PG main menu used in scripting buffers."
   (cons proof-general-name
 	(append
-	 proof-toolbar-scripting-menu
+	 (proof-toolbar-scripting-menu)
 	 proof-menu
 	 proof-config-menu
 	 (list proof-advanced-menu)
-	 (list proof-help-menu)))
-  "PG main menu used in scripting buffers.")
+	 (list proof-help-menu))))
 
-(defvar proof-aux-menu
+;;;###autoload
+(defun proof-aux-menu ()
+  "Construct and return PG auxiliary menu used in non-scripting buffers."
   (cons proof-general-name 
 	(append
-	 proof-toolbar-scripting-menu
+	 (proof-toolbar-scripting-menu)
 	 proof-config-menu
-	 (list proof-help-menu)))
-  "PG auxiliary menu used in non-scripting buffers.")
+	 (list proof-help-menu))))
 
 
 
@@ -493,9 +547,6 @@ without adjusting window layout."
 ;;; Favourites mechanism for prover-specific menu
 ;;;
 
-(defvar proof-menu-favourites nil
-  "The Proof General favourites menu for the current proof assistant.")
-
 (defun proof-menu-define-favourites-menu ()
   "Return menu generated from `PA-favourites'."
   (let ((favs (reverse (proof-ass favourites))) ents)
@@ -516,38 +567,6 @@ without adjusting window layout."
   
 ;;; Define stuff from favourites
 
-;;;###autoload
-(defmacro proof-defshortcut (fn string &optional key)
-  "Define shortcut function FN to insert STRING, optional keydef KEY.
-This is intended for defining proof assistant specific functions.
-STRING is inserted using `proof-insert', which see.
-KEY is added onto proof-assistant map."
-  `(progn
-     (if ,key
-	 (define-key (proof-ass keymap) (quote ,key) (quote ,fn)))
-     (defun ,fn ()
-       ,(concat "Shortcut command to insert " 
-		(replace-in-string  string "\\\\" "\\\\=") ;; for substitute-command-keys
-		" into the current buffer.\nThis simply calls `proof-insert', which see.")
-       (interactive)
-       (proof-insert ,string))))
-
-;;;###autoload
-(defmacro proof-definvisible (fn string &optional key)
-  "Define function FN to send STRING to proof assistant, optional keydef KEY.
-This is intended for defining proof assistant specific functions.
-STRING is sent using proof-shell-invisible-command, which see.
-KEY is added onto proof-assistant map."
-  `(progn
-     (if ,key
-	 (define-key (proof-ass keymap) (quote ,key) (quote ,fn)))
-     (defun ,fn ()
-       ,(concat "Command to send " 
-		(replace-in-string  string "\\\\" "\\\\=") ;; for substitute-command-keys
-		" to the proof assistant.")
-       (interactive)
-       (proof-shell-invisible-command ,string))))
-
 (defun proof-def-favourite (command inscript menuname &optional key new)
   "Define and a \"favourite\" proof assisant function.
 See doc of `proof-add-favourite' for first four arguments.
@@ -651,13 +670,6 @@ KEY is the optional key binding."
 ;;; Proof assistant settings mechanism.
 ;;;
 
-(defvar proof-assistant-settings nil
- "A list of default values kept in Proof General for current proof assistant.
-A list of lists (SYMBOL SETTING TYPE) where SETTING is a string value
-to send to the proof assistant using the value of SYMBOL and 
-and the function `proof-assistant-format'.  The TYPE item determines
-the form of the menu entry for the setting.")
-
 (defvar proof-menu-settings nil
   "Settings submenu for Proof General.")
 
@@ -671,6 +683,8 @@ the form of the menu entry for the setting.")
 			 ["Save Settings" (proof-settings-save)
 			  (proof-settings-changed-from-saved-p)]))
 	    ents)
+	;; TODO: for a large number of settings, we could generate
+	;; sub-menus according to the group.  
 	(while setgs
 	  (setq ents (cons 
 		      (apply 'proof-menu-entry-for-setting (car setgs)) ents))
@@ -681,9 +695,16 @@ the form of the menu entry for the setting.")
 
 (defun proof-menu-entry-name (symbol)
   "Return a nice menu entry name for SYMBOL."
-  (upcase-initials 
-   (replace-in-string (symbol-name symbol) "-" " ")))
-
+  ;; NB: for grouped settings, there is a pggroup symbol property and
+  ;; by convention the name of the setting begins with grp:
+  ;; We strip the group name from the menu entry name.
+  (let ((grp (get symbol 'pggroup))
+	(nm  (symbol-name symbol)))
+    (upcase-initials
+     (replace-in-string 
+      (if grp (replace-in-string nm (concat (downcase grp) ":") "") nm)
+      "-" " "))))
+	
 (defun proof-menu-entry-for-setting (symbol setting type)
   (let ((entry-name  (proof-menu-entry-name symbol))
 	(pasym	     (proof-ass-symv symbol)))
@@ -725,7 +746,6 @@ the form of the menu entry for the setting.")
   (interactive)
   (apply 'pg-custom-reset-vars (proof-settings-vars)))
 
-
 ;;; autoload for compiled version: used in macro proof-defpacustom
 ;;;###autoload
 (defun proof-defpacustom-fn (name val args)
@@ -739,15 +759,29 @@ the form of the menu entry for the setting.")
        ((eq (car args) :eval)
 	(setq evalform (cadr args))
 	(setq args (cdr args)))
+       ((eq (car args) :pgipcmd)
+	;; Construct a function which yields a PGIP string 
+	(setq setting `(lambda (x) 
+			  (pg-pgip-string-of-command (proof-assistant-format ,(cadr args) x))))
+	(setq args (cdr args)))
+       ((eq (car args) :pggroup) 
+	;; use the group as a prefix to the name, and set a pggroup property on it
+	(setq name (intern (concat (downcase (cadr args)) ":" (symbol-name name))))
+	(put name 'pggroup (cadr args))
+	(setq args (cdr args)))
        ((eq (car args) :type)
 	(setq type (cadr args))
-	(setq newargs (cons (car args) newargs)))
+	(setq args (cdr args))
+	(setq newargs (cons type (cons :type newargs))))
        (t
 	(setq newargs (cons (car args) newargs))))
       (setq args (cdr args)))
     (setq newargs (reverse newargs))
-    (unless (or setting evalform)
-      (error "defpacustom: missing :setting or :eval keyword"))
+    ;; PG 3.5 patch 22.4.03: allow empty :setting, :eval,
+    ;; because it's handy to put stuff on settings menu but
+    ;; inspect the settings elsewhere in code.
+    ;; (unless (or setting evalform)
+    ;; (error "defpacustom: missing :setting or :eval keyword"))
     (unless (and type
 		  (or (eq (eval type) 'boolean) 
 		      (eq (eval type) 'integer) 
@@ -775,14 +809,16 @@ the form of the menu entry for the setting.")
 	,@newargs
 	:set 'proof-set-value
 	:group 'proof-assistant-setting))
-    (if evalform
-	(eval
-	 `(defpgfun ,name ()
-	    ,evalform))
+    (cond
+     (evalform
+      (eval
+       `(defpgfun ,name ()
+	  ,evalform)))
+     (setting
       (eval
        `(defpgfun ,name ()
 	  (proof-assistant-invisible-command-ifposs
-	   (proof-assistant-settings-cmd (quote ,name))))))
+	   (proof-assistant-settings-cmd (quote ,name)))))))
     (setq proof-assistant-settings
 	  (cons (list name setting (eval type)) 
 		(remassoc name proof-assistant-settings)))))
@@ -793,10 +829,14 @@ the form of the menu entry for the setting.")
 NAME can correspond to some internal setting, flag, etc, for the
 proof assistant, in which case a :setting and :type value should be provided.
 The :type of NAME should be one of 'integer, 'boolean, 'string.
-The customization variable is automatically in group `proof-assistant-setting.
+The customization variable is automatically in group `proof-assistant-setting'.
 The function `proof-assistant-format' is used to format VAL.
 If NAME corresponds instead to a PG internal setting, then a form :eval to
 evaluate can be provided instead."
+  (eval-when-compile
+    (if (boundp 'proof-assistant-symbol)
+	;; declare variable to compiler to prevent warnings
+	(eval `(defvar ,(proof-ass-sym name) nil "Dummy for compilation."))))
   `(proof-defpacustom-fn (quote ,name) (quote ,val) (quote ,args)))
 
 (defun proof-assistant-invisible-command-ifposs (cmd)
@@ -813,7 +853,10 @@ evaluate can be provided instead."
 	;; FIXME: should only do if goals display is active,
 	;; messy otherwise. 
 	;; (we need a new flag for "active goals display").  
-	(if proof-showproof-command 
+	;; PG 3.5 (patch 22.04.04):
+	;; Let's approximate that by looking at proof-nesting-depth.
+	(if (and proof-showproof-command 
+		 (> proof-nesting-depth 0))
 	    (proof-shell-invisible-command proof-showproof-command))
 	;;  Could also repeat last command if non-state destroying.
 	)))
@@ -842,23 +885,6 @@ NB: if no settings are configured, this has no effect."
 	(apply 'concat (mapcar evalifneeded 
 			       proof-assistant-settings)))))
 
-(defun proof-assistant-format (string curvalue)
-  "Replace a format characters %b %i %s in STRING by formatted CURVALUE.
-Formatting suitable for current proof assistant, controlled by
-`proof-assistant-format-table' which see.
-Finally, apply `proof-assistant-setting-format' if non-nil.
-As a special case for boolean settings: the setting STRING 
-can be a cons cell of two strings, the first one for true (non-nil
-value) and the second for false."
-  (let ((setting
-	 (if (consp string)
-	     (if curvalue (car string) (cdr string))
-	   ;; Otherwise must use % format characters
-	   (proof-format proof-assistant-format-table string))))
-    (if proof-assistant-setting-format
-	(funcall proof-assistant-setting-format setting)
-      setting)))
-
 (defvar proof-assistant-format-table 
   (list
    (cons "%b" '(proof-assistant-format-bool curvalue))
@@ -876,6 +902,31 @@ NB: variable curvalue is dynamically scoped (used in proof-assistant-format).")
 (defun proof-assistant-format-string (value)
   (funcall proof-assistant-format-string-fn value))
 
+(defun proof-assistant-format (string curvalue)
+  "Replace a format characters %b %i %s in STRING by formatted CURVALUE.
+Formatting suitable for current proof assistant, controlled by
+`proof-assistant-format-table' which see.
+Finally, apply `proof-assistant-setting-format' if non-nil.
+Alternatively, STRING can be a function which yields a string when applied
+to the CURVALUE.
+As another special case for boolean settings: the setting STRING 
+can be a cons cell of two strings, the first one for true (non-nil
+value) and the second for false."
+  (let ((setting
+	 (cond
+	  ((stringp string)   ;; use % format characters
+	   (proof-format proof-assistant-format-table string))
+	  ((functionp string) ;; call the function
+	   (funcall string curvalue))
+	  ((consp string)     ;; true/false options
+	   (if curvalue (car string) (cdr string)))
+	  (t ;; no idea what to do
+	   (error "proof-assistant-format: called with invalid string arg %s" string)))))
+    (if proof-assistant-setting-format
+	(funcall proof-assistant-setting-format setting)
+      setting)))
+
+
 
 
 
