@@ -1,22 +1,25 @@
 ;; proof-splash.el -- Splash welcome screen for Proof General
 ;;
-;; Copyright (C) 1998-2001 LFCS Edinburgh. 
+;; Copyright (C) 1998-2005 LFCS Edinburgh. 
 ;; Author:    David Aspinall
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-splash.el,v 8.0 2004/04/17 23:39:59 da Exp
+;; proof-splash.el,v 9.0 2008/01/30 15:22:16 da Exp
 ;;
 ;;
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Customization of splash screen (was in proof-config)
+(require 'proof-site)
+
+;;
+;; Customization of splash screen 
+;;
 
 (defcustom proof-splash-enable t
   "*If non-nil, display a splash screen when Proof General is loaded."
   :type 'boolean
   :group 'proof-user-options)
 
-(defcustom proof-splash-time 2
+(defcustom proof-splash-time 3
   "Minimum number of seconds to display splash screen for.
 The splash screen may be displayed for a wee while longer than
 this, depending on how long it takes the machine to initialise 
@@ -36,19 +39,15 @@ Proof General."
     "Welcome to"
     (concat proof-assistant " Proof General!")
     nil
-    (substring proof-general-version
-	       (string-match "Version [^ ]+ " 
-			     proof-general-version)
-	       (match-end 0))
+    (concat "Version " proof-general-short-version ".")
     nil
-    "(C) LFCS, University of Edinburgh, 2004."
+    (concat "(C) LFCS, University of Edinburgh " proof-general-version-year)
     nil
     nil
-"    Please send problems and suggestions to da+pg-support@inf.ed.ac.uk, 
-     or use the menu command Proof-General -> Submit bug report."
+"    Please report problems at http://proofgeneral.inf.ed.ac.uk/trac
+    Visit the Proof General wiki at http://proofgeneral.inf.ed.ac.uk/wiki"
     nil
-    (unless (or proof-running-on-XEmacs proof-running-on-Emacs21)
-     "For a better Proof General experience, please use XEmacs or Emacs 21.X"))
+    "Find out more about Emacs via the Help menu.")
   "Evaluated to configure splash screen displayed when entering Proof General.
 A list of the screen contents.  If an element is a string or an image
 specifier, it is displayed centred on the window on its own line.  
@@ -73,17 +72,17 @@ If it is nil, a new line is inserted."
 
 
 ;; Compatibility between Emacs/XEmacs.
-(if (string-match "XEmacs" emacs-version)
-  ;; Constant nil function
-  (defun proof-emacs-imagep (img)
-    "See if IMG is an Emacs 21 image descriptor (returns nil since not E21)."
-    nil)
-  (defun proof-emacs-imagep (img)
-    "See if IMG is an Emacs 21 image descriptor."
-    (and (listp img) (eq (car img) 'image))))
+(eval-and-compile
+  (if (featurep 'xemacs)
+      ;; Constant nil function
+      (defsubst proof-emacs-imagep (img)
+	"See if IMG is an Emacs image descriptor (returns nil here on XEmacs)."
+	nil)
+    (defsubst proof-emacs-imagep (img)
+      "See if IMG is an Emacs image descriptor."
+      (and (listp img) (eq (car img) 'image)))))
 
 
-;; could be in proof-utils 
 (defun proof-get-image (name &optional nojpeg default)
   "Construct an image instantiator for an image, or string failing that.
 Different formats are chosen from according to what can be displayed.
@@ -93,42 +92,29 @@ DEFAULT gives return value in case image not valid."
   (let ((jpg (vector 'jpeg :file
 		     (concat proof-images-directory name ".jpg")))
 	(gif (vector 'gif :file
-		     (concat proof-images-directory 
-			     name
-			     (or (and
-				  (fboundp 'device-pixel-depth)
-				  (not (null (device-pixel-depth)))
-				  (> (device-pixel-depth) 8)
-				  ".gif")
-				 ;; Low colour gif for poor displays
-				 ".8bit.gif"))))
-	(xpm (vector 'xpm :file
-		     (concat proof-images-directory name ".xpm")))
+		     (concat proof-images-directory ".gif")))
 	(validfn (lambda (inst)
-		   (and (valid-instantiator-p inst 'image)
+		   (and (featurep 'xemacs)
+			(valid-instantiator-p inst 'image)
 			(file-readable-p (aref inst 2)))))
 	img)
   (cond
-   ((and window-system proof-running-on-XEmacs (featurep 'jpeg) (not nojpeg)
+   ((and (featurep 'xemacs) window-system
+	 (featurep 'jpeg) (not nojpeg)
 	 (funcall validfn jpg))
     jpg)
-   ((and window-system proof-running-on-XEmacs (featurep 'gif) (funcall validfn gif))
+   ((and (featurep 'xemacs) window-system
+	 (featurep 'gif) (funcall validfn gif))
     gif)
-   ((and window-system proof-running-on-XEmacs (featurep 'xpm) (funcall validfn xpm))
-    xpm)
-   ;; Support GNU Emacs 21
    ((and
-     proof-running-on-Emacs21
-     window-system
-     (setq img
+     (not (featurep 'xemacs)) window-system
+     (setq img 
 	   (find-image
 	    (list
 	     (list :type 'jpeg
 		   :file (concat proof-images-directory name ".jpg"))
 	     (list :type 'gif
-		   :file (concat proof-images-directory name ".gif"))
-	     (list :type 'xpm
-		   :file (concat proof-images-directory name ".xpm"))))))
+		   :file (concat proof-images-directory name ".gif"))))))
     img)
    (t
     (or default (concat "[ image " name " ]"))))))
@@ -146,7 +132,7 @@ Borrowed from startup-center-spaces."
 	 (fill-area-width  (* avg-pixwidth (- fill-column left-margin)))
 	 (glyph-pixwidth   (cond ((stringp glyph) 
 				  (* avg-pixwidth (length glyph)))
-				 ((and (fboundp 'glyphp)
+				 ((and (featurep 'xemacs)
 				       (glyphp glyph))
 				  (glyph-width glyph))
 				 ((proof-emacs-imagep glyph)
@@ -169,8 +155,8 @@ Borrowed from startup-center-spaces."
 
 (defun proof-splash-remove-screen (&optional nothing)
   "Remove splash screen and restore window config."
-  (let
-      ((splashbuf (get-buffer proof-splash-welcome)))
+  (let ((splashbuf (get-buffer proof-splash-welcome)))
+    (proof-splash-unset-frame-titles)
     (if (and 
 	 splashbuf
 	 proof-splash-timeout-conf)
@@ -180,10 +166,9 @@ Borrowed from startup-center-spaces."
 	      (progn
 		(if (cdr proof-splash-timeout-conf)
 		    (set-window-configuration (cdr proof-splash-timeout-conf)))
-		(if proof-running-on-XEmacs
+		(if (featurep 'xemacs)
 		    (redraw-frame nil t))))
-	  ;; Indicate that we have removed splash screen;
-	  ;; disable the timeout
+	  ;; Indicate removed splash screen; disable timeout
 	  (disable-timeout (car proof-splash-timeout-conf))
 	  (setq proof-splash-timeout-conf nil)
 	  (proof-splash-remove-buffer)))))
@@ -237,7 +222,7 @@ Otherwise, timeout inside this function after 10 seconds or so."
       (while splash-contents
 	(setq s (car splash-contents))
 	(cond
-	 ((and proof-running-on-XEmacs
+	 ((and (featurep 'xemacs)
 	       (vectorp s)
 	       (valid-instantiator-p s 'image))
 	  (let ((gly (make-glyph s)))
@@ -286,9 +271,12 @@ Otherwise, timeout inside this function after 10 seconds or so."
 (defun proof-splash-message ()
   "Make sure the user gets welcomed one way or another."
   (interactive)
-  (unless (or proof-splash-seen (noninteractive))
+  (unless (or proof-splash-seen noninteractive)
     (if proof-splash-enable
-	(proof-splash-display-screen (not (interactive-p)))
+	(progn
+	  ;; disable ordinary emacs splash
+	  (setq inhibit-startup-message t)
+	  (proof-splash-display-screen (not (interactive-p))))
       ;; Otherwise, a message
       (message "Welcome to %s Proof General!" proof-assistant))
     (setq proof-splash-seen t)))
@@ -297,28 +285,33 @@ Otherwise, timeout inside this function after 10 seconds or so."
   "Wait for proof-splash-timeout or input, then remove self from hook."
   (while (and proof-splash-timeout-conf ;; timeout still active
 	      (not (input-pending-p)))
-    (if proof-running-on-XEmacs
+    (if (featurep 'xemacs)
 	(sit-for 0 t)			; XEmacs: wait without redisplay
       ; (sit-for 1 0 t)))		; FSF: NODISP arg seems broken
       (sit-for 0)))
   (if proof-splash-timeout-conf         ;; not removed yet
       (proof-splash-remove-screen))
-  (if (and (input-pending-p)
-	   (fboundp 'next-command-event)) ; 3.3: this function
-					  ; disappeared from emacs, sigh
-      (setq unread-command-events
-	    (cons (next-command-event) unread-command-events)))
+  (if (fboundp 'next-command-event) ; 3.3: Emacs removed this
+      (if (input-pending-p)
+	  (setq unread-command-events
+		(cons (next-command-event) unread-command-events))))
   (remove-hook 'proof-mode-hook 'proof-splash-timeout-waiter))
 
+(defvar proof-splash-old-frame-title-format nil)
+
 (defun proof-splash-set-frame-titles ()
-  (let 
-      ((instance-name (concat
-		       (if (and proof-assistant
-				(not (string-equal proof-assistant "")))
+  (let ((instance-name (concat
+			(if (not (zerop (length proof-assistant)))
 			   (concat proof-assistant " "))
 		       "Proof General")))
+    (setq proof-splash-old-frame-title-format frame-title-format)
     (setq frame-title-format 
 	  (concat instance-name ":   %b"))))
 
+(defun proof-splash-unset-frame-titles ()
+  (when proof-splash-old-frame-title-format
+    (setq frame-title-format proof-splash-old-frame-title-format)
+    (setq proof-splash-old-frame-title-format nil)))
+
 (provide 'proof-splash)
 ;; End of proof-splash.
