@@ -5,17 +5,20 @@
 ;;	      Thomas Kleymann, Dilip Sequiera
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-syntax.el,v 8.0 2004/04/17 23:39:59 da Exp
+;; proof-syntax.el,v 9.0 2008/01/30 15:22:16 da Exp
 ;;
 
 (require 'font-lock)
-(require 'proof-config)
+(require 'proof-config)			; proof-case-fold-search
+(require 'proof-compat)			; proof-buffer-syntactic-context
 
-;; FIXME da: would regexp-opt be better here?  Or maybe
+;; TODO da: would regexp-opt be better here?  Or maybe
 ;;  (concat "\\<" (regexp-opt l) "\\>")
 (defun proof-ids-to-regexp (l)
   "Maps a non-empty list of tokens `l' to a regexp matching any element"
   (mapconcat (lambda (s) (concat "\\<" s "\\>")) l "\\|"))
+;; Stefan Monnier suggests:
+;;  (concat "\\<\\(?:" (mapconcat 'identity l "\\|") "\\)\\>"))
 
 (defun proof-anchor-regexp (e)
   "Anchor (\\`) and group the regexp E."
@@ -57,6 +60,18 @@ nil if a region cannot be found."
 ;; value of proof-case-fold-search.  Last arg to string-match is not
 ;; applicable.
 
+(defun proof-search-forward (string &optional bound noerror count)
+  "Like search-forward, but set case-fold-search to proof-case-fold-search."
+  (let
+      ((case-fold-search proof-case-fold-search))
+    (search-forward string bound noerror count)))
+
+(defun proof-replace-regexp-in-string (regexp rep string)
+  "Like replace-regexp-in-string, but set case-fold-search to proof-case-fold-search."
+  (let
+      ((case-fold-search proof-case-fold-search))
+    (replace-regexp-in-string regexp rep string)))
+
 (defun proof-re-search-forward (regexp &optional bound noerror count)
   "Like re-search-forward, but set case-fold-search to proof-case-fold-search."
   (let
@@ -101,23 +116,28 @@ nil if a region cannot be found."
 If so, return non-nil."
   (or
    (proof-buffer-syntactic-context)
-   (proof-looking-at-safe proof-script-comment-start-regexp)
-   (proof-looking-at-safe proof-string-start-regexp)))
+   (when (proof-looking-at-safe proof-script-comment-start-regexp) 'comment)
+   (when (proof-looking-at-safe proof-string-start-regexp) 'string)))
 
 
 
 ;; Replacing matches
 
-(defun proof-replace-string (string to-string)
-  "Non-interactive version of `replace-string', which see."
-  (while (search-forward string nil t)
+(defsubst proof-replace-string (string to-string)
+  "Non-interactive `replace-string', using `proof-case-fold-search'."
+  (while (proof-search-forward string nil t)
     (replace-match to-string nil t)))
 
-(defun proof-replace-regexp (regexp to-string)
-  "Non-interactive version of `replace-regexp', which see."
-  (while (re-search-forward regexp nil t)
+(defsubst proof-replace-regexp (regexp to-string)
+  "Non-interactive `replace-regexp', using `proof-case-fold-search'."
+  (while (proof-re-search-forward regexp nil t)
     (replace-match to-string nil nil)))
 
+(defsubst proof-replace-regexp-nocasefold (regexp to-string)
+  "Non-interactive `replace-regexp', forcing `case-fold-search' to nil."
+  (let ((case-fold-search nil))
+    (while (proof-re-search-forward regexp nil t)
+      (replace-match to-string nil nil))))
 
 ;; Generic font-lock
 
@@ -143,7 +163,7 @@ Default is comma separated, or SEPREGEXP if set."
 (defun proof-zap-commas (limit)
   "Remove the face of all `,' from point to LIMIT.
 Meant to be used from `font-lock-keywords'."
-  (while (search-forward "," limit t)
+  (while (proof-search-forward "," limit t)
     (if (memq (get-text-property (1- (point)) 'face)
 	      '(proof-declaration-name-face
 		font-lock-variable-name-face
@@ -262,6 +282,7 @@ Any other %-prefixed character inserts itself."
     (if acc (insert acc))
     (if pos (goto-char pos))))
 
+;;;###autoload
 (defun proof-splice-separator (sep strings)
   "Splice SEP into list of STRINGS."
   (let (stringsep)
