@@ -4,17 +4,8 @@
 ;; Author:    David Aspinall <David.Aspinall@ed.ac.uk>
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; proof-toolbar.el,v 8.0 2004/04/17 23:39:59 da Exp
+;; proof-toolbar.el,v 9.0 2008/01/30 15:22:17 da Exp
 ;;
-;; NB: FSF GNU Emacs has no toolbar facility. This file defines
-;; proof-toolbar-menu which holds the same commands and is put on the
-;; menubar by proof-toolbar-setup (perhaps surprisingly).
-;; Could consider moving the generic table stuff to proof-menu now.
-;;
-;; Toolbar is just for the scripting buffer, currently.
-;;
-;;
-;; TODO (minor things):
 ;;
 ;; 1. edit-toolbar cannot edit proof toolbar (even in a proof mode)
 ;; Need a variable containing  a specifier or similar.
@@ -31,25 +22,19 @@
 ;; 3. We could consider automatically disabling buttons which are
 ;; not configured for the prover, e.g. if proof-info-command is
 ;; not set, then the Info button should not be shown. 
-
-
-;;; IMPORT declaration  (only to suppress warnings for byte compile)
-;;; NB: can't put require proof-script here: leads to circular
-;;; requirement via proof-menu.
-;; (require 'proof-script)
-;; (autoload 'proof-shell-live-buffer "proof-shell")
-;; (autoload 'proof-shell-restart "proof-shell")
-
-
-(require 'proof-config)			; for <PA>-toolbar-entries
-
 ;;
-;; See `proof-toolbar-entries-default' and
-;;     `<PA>-toolbar-entries' in proof-config
-;; for the default generic toolbar and 
+;; See `proof-toolbar-entries-default' and `<PA>-toolbar-entries' 
+;; in pg-custom for the default generic toolbar and 
 ;; the per-prover toolbar contents variable.
 ;;
 
+(eval-when-compile
+  (require 'proof-utils))
+
+(require 'span)
+(require 'proof-config)
+(require 'proof-autoloads)
+
 ;;
 ;; Function, icon, button names
 ;; 
@@ -115,7 +100,7 @@
 		     (message "Button \"%s\" disabled" ,menuname))))
 	  buttonfnwe)))
     (if tooltip	;; no tooltip means menu-only item
-	(if proof-running-on-XEmacs
+	(if (featurep 'xemacs)
 	    (list (vector icon actualfn enableritem tooltip))
 	  (list (append (list icon actualfn token
 			      :help tooltip)
@@ -141,26 +126,24 @@ If `proof-toolbar-enable' is nil, change the current buffer toolbar
 to the default toolbar."
   (interactive)
   (if
-   (and ;; Check support in Emacs
+   (and ;; Check toolbar support...
+    window-system
     (or (and (featurep 'tool-bar)	; GNU Emacs tool-bar library
-	     (member 'xpm image-types))	; and XPM support
+	     (or (image-type-available-p 'xpm)   ;; and XPM
+		 (image-type-available-p 'png))) ;; or PNG
 	(and (featurep 'toolbar)	; or XEmacs toolbar library
-	     (featurep 'xpm)))		; and XPM support
-    ;; Check support in Window system
-    (memq (if proof-running-on-XEmacs (console-type) window-system)
-	  '(x mswindows gtk)))
+	     (featurep 'xpm))))		; and XPM support
 
       ;; Toolbar support is possible.  
       (progn
 	;; Check the toolbar has been built.
 	(or proof-toolbar (proof-toolbar-build))
 
-	;; Now see if user wants toolbar
-	;; or not (this can be changed dyamically).
+	;; Now see if user wants toolbar (this can be changed dyamically).
 	(if proof-toolbar-enable
 
 	    ;; Enable the toolbar in this buffer
-	    (if proof-running-on-Emacs21
+	    (if (not (featurep 'xemacs))
 		;; For GNU Emacs, we make a local tool-bar-map
 		(set (make-local-variable 'tool-bar-map) proof-toolbar)
 	      
@@ -170,7 +153,7 @@ to the default toolbar."
 	      (proof-toolbar-setup-refresh))
 
 	  ;; Disable the toolbar in this buffer
-	  (if proof-running-on-Emacs21
+	  (if (not (featurep 'xemacs))
 	      ;; For GNU Emacs, we remove local value of tool-bar-map
 	      (kill-local-variable 'tool-bar-map)
 	    ;; For XEmacs, we remove specifier and disable refresh.
@@ -183,57 +166,48 @@ to the default toolbar."
 (defun proof-toolbar-build ()
   "Build proof-toolbar."
   (let 
-      ((icontype (if (and (boundp 'device-pixel-depth)
-			  (< (device-pixel-depth) 16))
-		     ;; Select 8bit xpm's if we've got a 
-		     ;; limited colour depth.
-		     ".8bit.xpm" ".xpm"))
+      ((icontype ".xpm")
 
-       (proof-toolbar-icon-list
 	;; List of icon variable names and their associated image
 	;; files.  A list of lists of the form (VAR IMAGE).  IMAGE is
-	;; the root name for ;an image file in proof-images-directory.
-	;; The toolbar code expects to find files IMAGE.xpm or
-	;; IMAGE.8bit.xpm and chooses the best one for the display
-	;; properites.
+	;; the root name for a file in proof-images-directory.  The
+	;; toolbar code expects to find files IMAGE.xpm
+       (proof-toolbar-icon-list
 	(apply 'append
 	       (mapcar 'proof-toolbar-make-icon 
 		       (proof-ass toolbar-entries))))
 
-       (proof-toolbar-button-list
 	;; A toolbar descriptor evaluated in proof-toolbar-setup.
 	;; Specifically, a list of sexps which evaluate to entries in
-	;; a toolbar descriptor.  The default value
-	;; proof-toolbar-default-button-list will work for any proof
-	;; assistant.
+	;; a toolbar descriptor.  The default
+	;; `proof-toolbar-default-button-list' works for any prover.
+       (proof-toolbar-button-list
 	(append
 	 (apply 'append (mapcar 'proof-toolbar-make-toolbar-item 
 				(proof-ass toolbar-entries)))
-	 (if proof-running-on-XEmacs
+	 (if (featurep 'xemacs)
 	     (list [:style 3d])))))
 
-    ;; First set the button variables to glyphs.  
-    ;; (NB: this is a bit long-winded).
+    ;; First set the button variables to glyphs (bit long-windedly).  
     (mapcar
      (lambda (buttons)
        (let ((var	(car buttons))
 	     (iconfiles 
 	      (mapcar (lambda (name)
-			(concat proof-images-directory
-				"pg-" ;; added 1.3.04 to avoid Emacs clashes
+			(concat proof-images-directory "pg-"
 				name
 				icontype)) (cdr buttons))))
 	 (set var 
-	      (if proof-running-on-XEmacs
+	      (if (featurep 'xemacs)
 		  ;; On XEmacs, icon variable holds a list of glyphs
 		  (toolbar-make-button-list iconfiles)
-		;; On GNU emacs, it holds a filename for the icon,
-		;; without path or extension.  Warning!  This
-		;; can lead to name clashes with other packages.
-		(concat "pg-" (eval (cadr buttons)))))))
+		;; On GNU Emacs, it holds a filename for the icon,
+		;; without path or extension.  Watch for clashes with
+		;; icons from other packages!
+		(concat "epg-" (eval (cadr buttons)))))))
      proof-toolbar-icon-list)
 
-    (if proof-running-on-XEmacs
+    (if (featurep 'xemacs)
 	;; For XEmacs, we evaluate the specifier.
 	(setq proof-toolbar (mapcar 'eval proof-toolbar-button-list))
 
@@ -255,6 +229,8 @@ to the default toolbar."
 
 ;; Action to take after altering proof-toolbar-enable
 (defalias 'proof-toolbar-enable 'proof-toolbar-setup)
+
+;;;###autoload (autoload 'proof-toolbar-toggle "proof-toolbar")
 (proof-deftoggle proof-toolbar-enable proof-toolbar-toggle)
 
 
@@ -265,8 +241,8 @@ to the default toolbar."
 
 (defun proof-toolbar-setup-refresh ()
   "Enable the XEmacs hackery to update the toolbar."
-  (if  proof-toolbar-use-button-enablers
-      (progn
+  (if (featurep 'xemacs) ; skip compilation on GNU Emacs
+      (when  proof-toolbar-use-button-enablers 
 	;; Set the callback for updating the enablers
 	(add-hook 'proof-state-change-hook 'proof-toolbar-refresh)
 	;; Also call it whenever text changes in this buffer,
@@ -286,10 +262,11 @@ to the default toolbar."
 
 (defun proof-toolbar-disable-refresh ()
   "Disable the XEmacs hackery to update the toolbar."
-  (remove-hook 'proof-state-change-hook 'proof-toolbar-refresh)
-  (remove-hook 'after-change-functions 'proof-toolbar-refresh)
-  (if proof-toolbar-itimer (delete-itimer proof-toolbar-itimer))
-  (setq proof-toolbar-itimer nil))
+  (when (featurep 'xemacs) ; skip compilation on GNU Emacs
+    (remove-hook 'proof-state-change-hook 'proof-toolbar-refresh)
+    (remove-hook 'after-change-functions 'proof-toolbar-refresh)
+    (if proof-toolbar-itimer (delete-itimer proof-toolbar-itimer))
+    (setq proof-toolbar-itimer nil)))
 
 (deflocal proof-toolbar-refresh-flag nil
   "Flag indicating that the toolbar should be refreshed.")
@@ -315,24 +292,26 @@ to the default toolbar."
   "Force refresh of toolbar display to re-evaluate enablers.
 This function needs to be called anytime that enablers may have 
 changed state."
-  (if ;; Be careful to only add to correct buffer, and if it's live
-       (buffer-live-p buf)
-      (let ((enabler-state (mapcar 'eval proof-toolbar-enablers)))
-	(if 
-	    (not (equal enabler-state proof-toolbar-enablers-last-state))
-	    (progn
-	      (setq proof-toolbar-enablers-last-state enabler-state)
-	      ;; The official way to do this should be 
-	      ;; (set-specifier-dirty-flag default-toolbar)
-	      ;; but it doesn't work, so we do what VM does instead,
-	      ;; removing and re-adding.  
-	      (remove-specifier default-toolbar buf)
-	      (set-specifier default-toolbar proof-toolbar buf)
-	      ;; We set the dirty flag as well just in case it helps...
-	      (set-specifier-dirty-flag default-toolbar)
-	      (setq proof-toolbar-refresh-flag nil))))
-    ;; Kill off this itimer if it's owning buffer has died
-    (delete-itimer current-itimer)))
+  (if (featurep 'xemacs)		; skip compilation on GNU Emacs
+      (if ;; Be careful to only add to correct buffer, and if it's live
+	  (buffer-live-p buf)
+	  (let ((enabler-state (mapcar 'eval proof-toolbar-enablers)))
+	    (if 
+		(not (equal enabler-state proof-toolbar-enablers-last-state))
+		(progn
+		  (setq proof-toolbar-enablers-last-state enabler-state)
+		  (when (featurep 'xemacs)
+		    ;; The official way to do this should be 
+		    ;; (set-specifier-dirty-flag default-toolbar)
+		    ;; but it doesn't work, so we do what VM does instead,
+		    ;; removing and re-adding.  
+		    (remove-specifier default-toolbar buf)
+		    (set-specifier default-toolbar proof-toolbar buf)
+		    ;; We set the dirty flag as well just in case it helps...
+		    (set-specifier-dirty-flag default-toolbar))
+		  (setq proof-toolbar-refresh-flag nil))))
+	;; Kill off this itimer if it's owning buffer has died
+	(delete-itimer current-itimer))))
 
 ;;
 ;; =================================================================
@@ -576,14 +555,16 @@ changed state."
 	    (list menuname fnval)
 	    (if enablep 
 		(list ':active (list (proof-toolbar-enabler token))))))))))
-   
-(defconst proof-toolbar-scripting-menu
-  ;; Toolbar contains commands to manipulate script and
-  ;; other handy stuff.  
+
+;;;###autoload
+(defun proof-toolbar-scripting-menu ()
+  "Menu made from the Proof General toolbar commands."
+  ;; Prevent evaluation too early here, otherwise this is called
+  ;; during compilation when loading files with expanded easy-menu-define
+  ;; (e.g. pg-response/proof-aux-menu call)
   (apply 'append
-	  (mapcar 'proof-toolbar-make-menu-item 
-		  (proof-ass toolbar-entries)))
-  "Menu made from the Proof General toolbar commands.")
+	 (mapcar 'proof-toolbar-make-menu-item 
+		 (proof-ass toolbar-entries))))
 
  
 ;; 
