@@ -3,7 +3,7 @@
 ;; Copyright (C) 1998-2002 LFCS Edinburgh
 ;; Author:    David Aspinall <David.Aspinall@ed.ac.uk>
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
-;; Id:	      proof-x-symbol.el,v 8.0 2004/04/17 23:39:59 da Exp
+;; Id:	      proof-x-symbol.el,v 9.0 2008/01/30 15:22:17 da Exp
 ;;
 ;; The X-Symbol package is at http://x-symbol.sourceforge.net/
 ;;
@@ -42,12 +42,11 @@
 ;;
 ;; =================================================================
 
-;; Current TODO here:
-;;
-;; -- Is it possible to remove setting of language in x-symbol-enable?
-;; -- Simplify proof-x-symbol-initialize
-;; -- Investigate font-lock errors (XEmacs 21.4.8):
-;;     Error caught in `font-lock-pre-idle-hook': (wrong-type-argument markerp nil)
+(eval-when-compile
+  (require 'proof-utils))		; proof-ass
+
+(require 'proof-config)			; variables 
+(require 'proof-autoloads)		; 
 
 
 (defvar proof-x-symbol-initialized nil
@@ -63,7 +62,7 @@
   "A test to see whether x-symbol support may be available."
   (and
    (or (featurep 'x-symbol-hooks)
-       (and window-system		; Not on a tty
+       (and window-system
 	    (progn
 	      ;; put bundled version on load path
 	      (setq load-path
@@ -83,8 +82,8 @@
   "Initialize x-symbol support for Proof General, if possible.
 If ERROR is non-nil, give error on failure, otherwise a warning."
   (interactive)
-  (unless (or proof-x-symbol-initialized ;; already done
-	      ;; or can't be done
+  (unless (or proof-x-symbol-initialized   ; already done
+	      (not proof-assistant-symbol) ; too early
 	      (not (proof-x-symbol-support-maybe-available)))
     (let*
 	((xs-lang        (proof-ass x-symbol-language))
@@ -128,23 +127,11 @@ The package is available at http://x-symbol.sourceforge.net/"))
 	(require 'x-symbol-vars)  ;; [new requirement to require this]
 	(let*
 	    ((xs-xtra-modes  proof-xsym-extra-modes)
-	     (xs-std-modes   
-	      (list
-	       ;; NB: there is a problem with initialization order
-	       ;; here, these variables are set in script/shell mode
-	       ;; initialization.  They ought to be set earlier, and
-	       ;; enforced as part of the generic scheme.  For the
-	       ;; time being, we use default constructed names [which
-	       ;; every prover should follow]
-	       (or proof-mode-for-shell
-		   (intern (concat assistant "-shell-mode")))
-	       (or proof-mode-for-response
-		   (intern (concat assistant "-response-mode")))
-	       (or proof-mode-for-script
-		   ;; FIXME: next one only correct for isabelle
-		   (intern (concat assistant "-proofscript-mode")))
-	       (or  proof-mode-for-goals
-		    (intern (concat assistant "-goals-mode")))))
+	     (xs-std-modes   (list
+			      proof-mode-for-shell
+			      proof-mode-for-response
+			      proof-mode-for-script
+			      proof-mode-for-goals))
 	     (all-xs-modes   (append xs-std-modes xs-xtra-modes))
 	     (am-entry       (list proof-xsym-extra-modes t 
 				   `(quote ,xs-lang)))
@@ -176,28 +163,28 @@ The package is available at http://x-symbol.sourceforge.net/"))
 ;;;###autoload
 (defun proof-x-symbol-enable ()
   "Turn on or off X-Symbol in current Proof General script buffer.
-This invokes `x-symbol-mode' to toggle the setting for the current
-buffer, and then sets PG's option for default to match.
-Also we arrange to have X-Symbol mode turn itself on automatically 
-in future if we have just activated it for this buffer."
-;; Calls proof-x-symbol-toggle-clean-buffers afterwards.
-  (if (not proof-x-symbol-initialized) ;; Check inited
-      (progn
-	(set (proof-ass-sym x-symbol-enable) nil) ; assume failure!
-	(proof-x-symbol-initialize 'giveerrors)
-	(set (proof-ass-sym x-symbol-enable) t)))
-  (x-symbol-mode)
-  (proof-x-symbol-mode-associated-buffers))
-
+This invokes `x-symbol-mode' to change the setting for the current
+buffer. "
+  (when (proof-ass x-symbol-enable)
+    (unless proof-x-symbol-initialized ;; Check inited
+      (set (proof-ass-sym x-symbol-enable) nil) ; assume failure!
+      (proof-x-symbol-initialize 'giveerrors)
+      (set (proof-ass-sym x-symbol-enable) t)))
+    
+  (when (and proof-x-symbol-initialized
+	     (fboundp 'x-symbol-mode))
+    (x-symbol-mode (if (proof-ass x-symbol-enable) 1 0))
+    (proof-x-symbol-mode-associated-buffers)))
+  
 ;; Old behaviour for proof-x-symbol-enable was to update state in all
 ;; buffers --- but this can take ages if there are many buffers!  
-;; We also used to refresh the output, but this doesn't always work.
+;; Refreshing output buffers also causes glitches
 ;; (proof-x-symbol-mode-all-buffers)
 ;; (proof-x-symbol-refresh-output-buffers))
 
 
 (defun proof-x-symbol-refresh-output-buffers ()
-  ;; FIXME: this isn't used.  Might be nice to do so again, turning
+  ;; NB: this isn't used.  Might be nice to do so again, turning
   ;; off X-Sym can leave junk displayed.  OTOH, sending messages to PA
   ;; can give errors, because there is no generic "refresh" or
   ;; "repeat" option.  (Isar: gives errors outside proof mode).
@@ -206,7 +193,7 @@ in future if we have just activated it for this buffer."
 This function is intended to clean the display after a change
 in the status of X-Symbol display.
 This is a subroutine of proof-x-symbol-enable."
-  (proof-shell-maybe-erase-response nil t t)
+  (pg-response-maybe-erase nil t t)
   (if (and proof-showproof-command (proof-shell-available-p))
     (proof-shell-invisible-command proof-showproof-command)))
 
@@ -214,14 +201,10 @@ This is a subroutine of proof-x-symbol-enable."
 (defun proof-x-symbol-mode-associated-buffers ()
   "Activate/deactivate x-symbols in all Proof General associated buffers.
 A subroutine of proof-x-symbol-enable."
-  ;; Response and goals buffer are fontified/decoded 
-  ;; manually in the code, configuration only sets
-  ;; x-symbol-language.
   (proof-map-buffers (list proof-goals-buffer 
 			   proof-response-buffer
 			   proof-trace-buffer)
    (proof-x-symbol-config-output-buffer))
-  ;; Shell has its own configuration
   (proof-with-current-buffer-if-exists proof-shell-buffer
    (proof-x-symbol-shell-config)))
 
@@ -233,7 +216,26 @@ A subroutine of proof-x-symbol-enable."
 ;; 
 
 ;;;###autoload
-(defalias 'proof-x-symbol-decode-region 'x-symbol-decode-region)
+(defun proof-x-symbol-decode-region (start end)
+  (let ((newend (x-symbol-decode-region start end)))
+    ;; subscripts in non-mule mode are done with fontification, see x-symbol-fontify
+    (if (featurep 'mule)
+	(let ((font-lock-keywords  x-symbol-font-lock-keywords) ;; TODO: compile keywords
+	      (font-lock-defaults '(x-symbol-font-lock-keywords t))
+	      ;; GE 21.1: the `font-lock-set-defaults' flag prevents font lock
+	      ;; complaining (in font-lock-compile-keywords) that
+	      ;; we've not run font-lock-set-defaults.  It gives dire
+	      ;; warnings what may happen otherwise but dynamic
+	      ;; binding of `font-lock-keywords' here should prevent
+	      ;; global changes to that variable.
+	      (font-lock-set-defaults t)
+	      (x-symbol-mode	t)
+	      (x-symbol-subscripts t))
+	  (condition-case err
+	      (font-lock-fontify-keywords-region start newend nil)
+	    (t (proof-debug "Caught condition %s in `font-lock-fontify-keywords-region'"
+			    (car err))))))))
+    
 
 (defun proof-x-symbol-encode-shell-input ()
   "Encode shell input in the variable STRING.
@@ -271,8 +273,7 @@ Assumes that the current buffer is the proof shell buffer."
   ;; NB: after changing X-Symbols in output it would be nice to
   ;; refresh display, but there's no robust way of doing that yet
   ;; (see proof-x-symbol-refresh-output-buffers above)
-  ;; [ Actually, we could ask that the activate/decativate command
-  ;;   itself does this ]
+  ;; [ The activate/decativate prover command itself could do it ]
   ;;
   (if proof-x-symbol-initialized
       (progn
@@ -307,46 +308,28 @@ Assumes that the current buffer is the proof shell buffer."
 ;;;###autoload
 (defun proof-x-symbol-config-output-buffer ()
   "Configure the current output buffer (goals/response/trace) for X-Symbol."
-  (if (proof-ass x-symbol-enable)
-      (progn
-	(proof-x-symbol-set-language)
-	;; BEGIN: Code below from x-symbol.el/x-symbol-mode-internal
-	(unless (or (not (boundp 'enable-multibyte-characters))
-		    (not (fboundp 'set-buffer-multibyte))
-		    enable-multibyte-characters)
-	  ;; Emacs: we need to convert the buffer from unibyte to multibyte
-	  ;; since we'll use multibyte support for the symbol charset.
-	  ;; TODO: try to do it less often
-	  (let ((modified (buffer-modified-p))
-		(inhibit-read-only t)
-		(inhibit-modification-hooks t))
-	    (unwind-protect
-		(progn
-		  (decode-coding-region (point-min) (point-max) 'undecided)
-		  (set-buffer-multibyte t))
-	      (set-buffer-modified-p modified))))
-	;; END code from x-symbol.el/x-symbol-mode-internal
-
-	;; If we're turning on x-symbol, attempt to convert to 
-	;; characters.  (Only works if the buffer already
-	;; contains tokens!)
-	(x-symbol-decode))))
-  ;; Encoding back to tokens doesn't work too well: needs to 
-  ;; do some de-fontification to remove font properties, and
-  ;; is flaky anyway because token -> char not nec injective.
-  ;  (if (boundp 'x-symbol-language)
-  ;	;; If we're turning off x-symbol, convert back to tokens.
-  ;     (x-symbol-encode))))
-
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;
-;; Initialize x-symbol-support on load-up if user has asked for it
-;;
-;; FIXME: this initialization seems to result in x-symbol-language not
-;; being set properly.  We let this be called on demand instead.
-;(if (proof-ass x-symbol-enable) 
-;      (proof-x-symbol-initialize))
+  (when (proof-ass x-symbol-enable)
+    (proof-x-symbol-set-language)
+    ;; BEGIN: Code below from x-symbol.el/x-symbol-mode-internal
+    (unless (or (not (boundp 'enable-multibyte-characters))
+		(not (fboundp 'set-buffer-multibyte))
+		enable-multibyte-characters)
+      ;; Emacs: we need to convert the buffer from unibyte to multibyte
+      ;; since we'll use multibyte support for the symbol charset.
+      ;; TODO: try to do it less often
+      (let ((modified (buffer-modified-p))
+	    (inhibit-read-only t)
+	    (inhibit-modification-hooks t))
+	(unwind-protect
+	    (progn
+	      (decode-coding-region (point-min) (point-max) 'undecided)
+	      (set-buffer-multibyte t))
+	  (set-buffer-modified-p modified))))
+    ;; END code from x-symbol.el/x-symbol-mode-internal
+    
+    ;; If we're turning on x-symbol, attempt to convert current contents.
+    ;; (reverse doesn't work so cleanly so we don't try it)
+    (proof-x-symbol-decode-region (point-min) (point-max))))
 
 (provide 'proof-x-symbol)
 ;; End of proof-x-symbol.el
