@@ -1,9 +1,9 @@
-;; isar.el	  Major mode for Isabelle/Isar proof assistant
+; isar.el --- Major mode for Isabelle/Isar proof assistant
 ;; Copyright (C) 1994-2004 LFCS Edinburgh.
 ;;
 ;; License:   GPL (GNU GENERAL PUBLIC LICENSE)
 ;;
-;; Maintainers:   David Aspinall, Stefan Berghofer
+;; Maintainers:   David Aspinall, Makarius, Stefan Berghofer
 ;;
 ;; Authors:       David Aspinall <David.Aspinall@ed.ac.uk>
 ;;   	          Markus Wenzel <wenzelm@in.tum.de>
@@ -11,16 +11,19 @@
 ;; Contributors:  David von Oheimb, Sebastian Skalberg
 ;;
 ;;
-;; isar.el,v 8.0 2004/04/17 23:40:00 da Exp
+;; isar.el,v 9.0 2008/01/30 15:22:19 da Exp
 ;;
 
+;;; Code:
 (require 'proof)
 
-;;
-;; Add generic code for Isabelle and Isabelle/Isar
-;;
-(setq load-path (cons (concat proof-home-directory "isa/") load-path))
-(require 'isabelle-system)
+(eval-when-compile
+  (require 'span)
+  (require 'proof-syntax)
+  (proof-ready-for-assistant 'isar))	; compile for isar
+
+(require 'isabelle-system)		; system code
+(require 'isar-find-theorems)		; "Find Theorems" search form
 
 ;;
 ;; Load syntax
@@ -35,7 +38,7 @@ See -k option for Isabelle interface script."
     ;; Pickup isar-keywords file from somewhere appropriate,
     ;; giving user chance to set name of file, or based on
     ;; name of logic.
-    (isabelle-load-isar-keywords 
+    (isabelle-load-isar-keywords
      (or isar-keywords-name
 	 isabelle-chosen-logic)))
 (require 'isar-syntax)
@@ -53,48 +56,9 @@ See -k option for Isabelle interface script."
 
 ;; Adjust toolbar entries (must be done before proof-toolbar is loaded).
 
-(if proof-running-on-XEmacs
-    (setq isar-toolbar-entries
-	  (remassoc 'qed (remassoc 'goal isar-toolbar-entries))))
-
-
-;;;
-;;; theory header
-;;;
-
-(defun isar-detect-header ()
-  "Detect new-style theory header in current buffer"
-  (let ((header-regexp (isar-ids-to-regexp '("header" "theory")))
-        (white-space-regexp "\\(\\s-\\|\n\\)+")
-        (cmt-end-regexp (regexp-quote proof-script-comment-end))
-        (cmt-start-regexp (regexp-quote proof-script-comment-start))
-        (found-header nil) forward-amount
-        (end (point-max)) (cont t) (cmt-level 0))
-    (save-excursion
-      (goto-char (point-min))
-      (while (and cont (< (point) end))
-        (setq forward-amount 1)
-        (cond
-         ;; comments
-         ((proof-looking-at cmt-start-regexp)
-          (setq forward-amount (length (match-string 0)))
-          (incf cmt-level))
-         ((proof-looking-at cmt-end-regexp)
-          (setq forward-amount (length (match-string 0)))
-          (decf cmt-level))
-         ((> cmt-level 0))
-         ;; white space
-         ((proof-looking-at white-space-regexp)
-          (setq forward-amount (length (match-string 0))))
-         ;; theory header
-         ((proof-looking-at header-regexp)
-          (setq found-header t)
-          (setq cont nil))
-         ;; bad stuff
-         (t
-          (setq cont nil)))
-        (and cont (forward-char forward-amount)))
-      found-header)))
+(eval-after-load "pg-custom"
+  '(setq isar-toolbar-entries
+	 (remassoc 'qed (remassoc 'goal isar-toolbar-entries))))
 
 
 (defun isar-strip-terminators ()
@@ -102,7 +66,7 @@ See -k option for Isabelle interface script."
   (interactive)
   (save-excursion
     (goto-char (point-min))
-    (while (search-forward ";" (point-max) t)
+    (while (proof-search-forward ";" (point-max) t)
       (if (not (proof-buffer-syntactic-context))
 	  (progn
 	    (delete-backward-char 1)
@@ -119,10 +83,10 @@ See -k option for Isabelle interface script."
   "Configure generic proof scripting mode variables for Isabelle/Isar."
   (setq
    proof-assistant-home-page    isar-web-page
-   proof-mode-for-script        'isar-mode
+   proof-prog-name		(isabelle-command-line)
    ;; proof script syntax
    proof-terminal-char          ?\;     ; forcibly ends a command
-   proof-script-command-start-regexp 
+   proof-script-command-start-regexp
    (proof-regexp-alt
     ;; FIXME: this gets { and } wrong: they must _not_ appear as {* *}
     ;; because that's lexically a kind of comment.
@@ -165,9 +129,14 @@ See -k option for Isabelle interface script."
    proof-context-command        "print_context"
    proof-info-command           "welcome"
    proof-kill-goal-command      "ProofGeneral.kill_proof"
-   proof-find-theorems-command  "thms_containing %s"
+;; da: TODO: we should set this from our own setting, not suggest 
+;; people customize prover-configuration variables
+;   proof-find-theorems-command  "find_theorems %s"               ;; minibuffer
+   proof-find-theorems-command  'isar-find-theorems-minibuffer ;; equivalent
+;   proof-find-theorems-command  'isar-find-theorems-form        ;; search form
    proof-shell-start-silent-cmd "disable_pr"
    proof-shell-stop-silent-cmd  "enable_pr"
+   proof-shell-trace-output-regexp  "\375\\|\^AV"
    ;; command hooks
    proof-goal-command-p         'isar-goal-command-p
    proof-really-save-command-p  'isar-global-save-command-p
@@ -175,23 +144,23 @@ See -k option for Isabelle interface script."
    proof-find-and-forget-fn     'isar-find-and-forget
    proof-state-preserving-p     'isar-state-preserving-p
    proof-shell-compute-new-files-list 'isar-shell-compute-new-files-list
-   ;; span menu 
+   ;; span menu
    proof-script-span-context-menu-extensions 'isabelle-create-span-menu))
 
-
 (defun isar-shell-mode-config-set-variables ()
   "Configure generic proof shell mode variables for Isabelle/Isar."
   (setq
-   pg-subterm-first-special-char        ?\350
+   pg-topterm-regexp			"\376\\|\^AW"
 
-   proof-shell-wakeup-char              ?\372
-   proof-shell-annotated-prompt-regexp  "^\\w*[>#] \372"
+   pg-topterm-goalhyplit-fn		'isar-goalhyplit-test
+   proof-shell-wakeup-char              nil
+   proof-shell-annotated-prompt-regexp  "^\\w*[>#] \372\\|^\\w*[>#] \^AS"
 
-   ;; This pattern is just for comint.
-   proof-shell-prompt-pattern           "^\\w*[>#] "
+   ;; just for comint.
+   proof-shell-prompt-pattern   "^\\w*[>#] "
 
    ;; for issuing command, not used to track cwd in any way.
-   proof-shell-cd-cmd                   (isar-markup-ml "ThyLoad.add_path \"%s\"")
+   proof-shell-cd-cmd		(isar-markup-ml "ThyLoad.add_path \"%s\"")
 
    ;; Escapes for filenames inside ML strings.
    ;; We also make a hack for a bug in Isabelle, by switching from
@@ -207,8 +176,8 @@ See -k option for Isabelle interface script."
      '(("\\\\" . "\\\\") ("\""   . "\\\"")))
 
    proof-shell-proof-completed-regexp   nil     ; n.a.
-   proof-shell-interrupt-regexp         "\364\\*\\*\\* Interrupt"
-   proof-shell-error-regexp             "\364\\*\\*\\*"
+   proof-shell-interrupt-regexp         "\364\\*\\*\\* Interrupt\\|\^AM\\*\\*\\* Interrupt"
+   proof-shell-error-regexp             "\364\\*\\*\\*\\|\^AM\\*\\*\\*"
    proof-shell-abort-goal-regexp        nil     ; n.a.
 
    pg-next-error-regexp	  "\\((line \\([0-9]+\\) of \"[^\"]+\")\\)"
@@ -216,44 +185,30 @@ See -k option for Isabelle interface script."
 
    ;; matches names of assumptions
    proof-shell-assumption-regexp        isar-id
-   ;; matches subgoal name
-   ;; da: not used at the moment, maybe after 4.0 used for
-   ;; proof-generic-goal-hyp-fn to get pg-goals-like features.
-   ;; proof-shell-goal-regexp           "\370[ \t]*\\([0-9]+\\)\\."
 
-   proof-shell-start-goals-regexp       "\366\n"
-   proof-shell-end-goals-regexp         "\367"
-   pg-topterm-char			?\370
+   proof-shell-start-goals-regexp       "\366\n\\|\^AO\n"
+   proof-shell-end-goals-regexp         "\367\\|\^AP"
 
-   ;; FIXME: next one is needed for backward compatibility.  
-   ;; Would be nice to remove this somehow else, it's only used for 
-   ;; Isar and pre-PGIP.  One way would be to hack the 
-   ;; (now obsolete) defpacustom calls.
-   proof-assistant-setting-format       
-   (unless isa-supports-pgip 'isar-markup-ml)
-
-   proof-shell-init-cmd			'(proof-assistant-settings-cmd)
+   proof-shell-init-cmd			nil
    proof-shell-restart-cmd              "ProofGeneral.restart"
 
    proof-shell-eager-annotation-start-length 1
-   proof-shell-eager-annotation-start   "\360\\|\362"
-   proof-shell-eager-annotation-end     "\361\\|\363"
-   ;; see isar-pre-shell-start for proof-shell-trace-output-regexp
+   proof-shell-eager-annotation-start   "\360\\|\362\\|\^AI\\|\^AK"
+   proof-shell-eager-annotation-end     "\361\\|\363\\|\^AJ\\|\^AL"
 
    ;; Isabelle is learning to talk PGIP...
    proof-shell-match-pgip-cmd		"<pgip"
-   proof-shell-issue-pgip-cmd		
-	(if isa-supports-pgip 'isabelle-process-pgip nil)
+   proof-shell-issue-pgip-cmd		'isabelle-process-pgip
 
    ;; Some messages delimited by eager annotations
    proof-shell-clear-response-regexp    "Proof General, please clear the response buffer."
    proof-shell-clear-goals-regexp       "Proof General, please clear the goals buffer."
 
    ;; Theorem dependencies.   NB: next regex anchored at end with eager annot end
-   proof-shell-theorem-dependency-list-regexp "Proof General, theorem dependencies of \\(.*\\) are \"\\(.*\\)\"\361"
+   proof-shell-theorem-dependency-list-regexp "Proof General, theorem dependencies of \\(.*\\) are \"\\(.*\\)\"\\(\361\\|\^AJ\\)"
    proof-shell-theorem-dependency-list-split "\" \""
    proof-shell-show-dependency-cmd "thm %s;"
-   proof-shell-identifier-under-mouse-cmd 
+   proof-shell-identifier-under-mouse-cmd
    '((nil     "thm %s;")
      (string  "term \"%s\";")
      (comment "term \"%s\";"))
@@ -261,18 +216,14 @@ See -k option for Isabelle interface script."
    ;; Allow font-locking for output based on hidden annotations, see
    ;; isar-output-font-lock-keywords-1
    pg-use-specials-for-fontify		t
+   pg-after-fontify-output-hook	        'pg-remove-specials
 
-   ;; === ANNOTATIONS  === these ones are not implemented in Isabelle
-   proof-shell-result-start             "\372 Pbp result \373"
-   proof-shell-result-end               "\372 End Pbp result \373"
-   pg-subterm-anns-use-stack            t
-   pg-subterm-start-char                ?\372
-   pg-subterm-sep-char                  ?\373
-   pg-subterm-end-char                  ?\374
-   pg-after-fontify-output-hook	
-   (if proof-experimental-features 
-       'isabelle-convert-idmarkup-to-subterm 'pg-remove-specials)
-   pg-subterm-help-cmd			"term %s" 
+   pg-special-char-regexp
+   (if proof-shell-unicode "[0-9A-Z]"
+     ;; next string could be: "[\350-\377]", but that's buggy with XEmacs 21.5 (beta)
+     "\\|\\|\\|\\|\\|\\|\\|\\|\\|8\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|\\|")
+
+   pg-subterm-help-cmd			"term %s"
 
    proof-cannot-reopen-processed-files  t
    proof-shell-process-file
@@ -349,7 +300,7 @@ proof-shell-retract-files-regexp."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;;
-;;; help menu  
+;;; help menu
 ;;;
 
 ;;; da: how about a `C-c C-a h ?' for listing available keys?
@@ -364,22 +315,49 @@ proof-shell-retract-files-regexp."
 (proof-definvisible isar-help-facts "print_facts" [h f])
 (proof-definvisible isar-help-syntax "print_syntax" [h i])
 (proof-definvisible isar-help-induct-rules "print_induct_rules" [h I])
-(proof-definvisible isar-help-intro-rules "print_intros" [h r])
 (proof-definvisible isar-help-methods "print_methods" [h m])
 (proof-definvisible isar-help-simpset "print_simpset" [h S])
 (proof-definvisible isar-help-binds "print_binds" [h b])
 (proof-definvisible isar-help-theorems "print_theorems" [h t])
 (proof-definvisible isar-help-trans-rules "print_trans_rules" [h T])
 
+;;
+;; Command menu
+;;
+
+;; NB: would be nice to query save of the buffer first for these
+;; next two: but only convenient emacs functions offer save for
+;; all buffers.
+(proof-definvisible isar-cmd-display-draft
+ '(format "display_drafts \"%s\"" buffer-file-name)
+ [(control d)])
+
+(proof-definvisible isar-cmd-print-draft
+  '(if (y-or-n-p
+	(format "Print draft of file %s ?" buffer-file-name))
+       (format "print_drafts \"%s\"" buffer-file-name)
+     (error "Aborted."))
+  [(control p)])
+
+(proof-definvisible isar-cmd-refute	"refute" [r])
+(proof-definvisible isar-cmd-quickcheck "quickcheck" [(control q)])
+(proof-definvisible isar-cmd-sledgehammer "sledgehammer" [(control s)])
+
 (defpgdefault menu-entries
   (append
    (list isabelle-logics-menu)
+   (list
+    (cons "Commands"
+          (list
+           ["refute"             isar-cmd-refute         t]
+           ["quickcheck"         isar-cmd-quickcheck     t]
+           ["sledgehammer"       isar-cmd-sledgehammer   t]
+	   ["display draft"	 isar-cmd-display-draft  t])))
    (list
     (cons "Show me ..."
           (list
            ["cases"              isar-help-cases          t]
            ["facts"              isar-help-facts          t]
-           ["matching rules"     isar-help-intro-rules    t]
            ["term bindings"      isar-help-binds          t]
 	   "----"
            ["classical rules"    isar-help-claset         t]
@@ -396,22 +374,23 @@ proof-shell-retract-files-regexp."
 
 ;; undo proof commands
 (defun isar-count-undos (span)
-  "Count number of undos in a span, return the command needed to undo that far."
+  "Count number of undos SPAN, return command needed to undo that far."
   (let
       ((case-fold-search nil)       ;FIXME ??
        (ct 0) str i)
     (while span
-      (setq str (span-property span 'cmd))
+      (setq str (or (span-property span 'cmd) ""))
       (cond ((eq (span-property span 'type) 'vanilla)
              (or (proof-string-match isar-undo-skip-regexp str)
 		 (proof-string-match isar-undo-ignore-regexp str)
                  (setq ct (+ 1 ct))))
-            ((eq (span-property span 'type) 'pbp)  ;FIXME dead code?
-             ;; this case probably redundant for Isabelle, unless
-             ;; we think of some nice ways of matching non-undoable
-             ;; commands.
+            ((eq (span-property span 'type) 'pbp)
+             ;; this case for automatically inserted text (e.g. sledgehammer)
              (cond ((not (proof-string-match isar-undo-skip-regexp str))
+		    (setq ct 1)
                     (setq i 0)
+		    ;; If we find a semicolon, assume several commands,
+		    ;; and increment the undo count.
                     (while (< i (length str))
                       (if (= (aref str i) proof-terminal-char)
                           (setq ct (+ 1 ct)))
@@ -425,12 +404,12 @@ proof-shell-retract-files-regexp."
   "Return commands to be used to forget SPAN."
   (let (str ans answers)
     (while span
-      (setq str (span-property span 'cmd))
+      (setq str (or (span-property span 'cmd) ""))
       (setq ans nil)
       (cond
        ;; comment, diagnostic, nested proof command: skip
        ;; (da: adding new span types may break this code,
-       ;;  ought to test for type 'cmd before looking at 
+       ;;  ought to test for type 'cmd before looking at
        ;;  str below)
        ;; FIXME: should adjust proof-nesting-depth here.
        ((or (eq (span-property span 'type) 'comment)
@@ -444,21 +423,15 @@ proof-shell-retract-files-regexp."
        ;; open goal: skip and exit
        ((proof-string-match isar-goal-command-regexp str)
         (setq span nil))
-       ;; not undoable: fail and exit 
-       ;; [da: this is an odd case: it issues cannot_undo command to Isar,
-       ;;  which immediately generates an error, I think it's a bit confusing
-       ;; for the user]
-       ((proof-string-match isar-undo-fail-regexp str)
+       ;; control command: cannot undo
+       ((and (proof-string-match isar-undo-fail-regexp str))
+        (setq ans (isar-cannot-undo (match-string 1 str)))
         (setq answers nil)
-        (setq ans (isar-cannot-undo str))
         (setq span nil))
        ;; theory: remove and exit
        ((proof-string-match isar-undo-remove-regexp str)
         (setq ans (isar-remove (match-string 2 str)))
         (setq span nil))
-       ;; context switch: kill
-       ((proof-string-match isar-undo-kill-regexp str)
-        (setq ans isar-kill))
        ;; else: undo
        (t
         (setq ans isar-undo)))
@@ -466,19 +439,18 @@ proof-shell-retract-files-regexp."
       (if span (setq span (next-span span 'type))))
     (if (null answers) proof-no-command (apply 'concat answers))))
 
-
-
-(defun isar-goal-command-p (str)
-  "Decide whether argument is a goal or not"
-  (proof-string-match isar-goal-command-regexp str))
+(defun isar-goal-command-p (span)
+  "Decide whether argument SPAN is a goal or not."
+  (proof-string-match isar-goal-command-regexp
+		      (or (span-property span 'cmd) "")))
 
 (defun isar-global-save-command-p (span str)
-  "Decide whether argument really is a global save command"
+  "Decide whether argument SPAN with command STR is a global save command."
   (or
    (proof-string-match isar-global-save-command-regexp str)
    (let ((ans nil) (lev 0) cmd)
      (while (and (not ans) span (setq span (prev-span span 'type)))
-       (setq cmd (span-property span 'cmd))
+       (setq cmd (or (span-property span 'cmd) ""))
        (cond
         ;; comment: skip
         ((eq (span-property span 'type) 'comment))
@@ -494,10 +466,10 @@ proof-shell-retract-files-regexp."
      (eq ans 'yes))))
 
 (defvar isar-current-goal 1
-  "Last goal that emacs looked at.")
+  "Last goal that Emacs looked at.")
 
 (defun isar-state-preserving-p (cmd)
-  "Non-nil if command preserves the proofstate."
+  "Non-nil if command CMD preserves the proofstate."
   (proof-string-match isar-undo-skip-regexp cmd))
 
 
@@ -505,25 +477,27 @@ proof-shell-retract-files-regexp."
 ;;   Commands specific to isar                                      ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(proof-defshortcut isar-super    "\\<^sup>" [(control u)])
-(proof-defshortcut isar-sub      "\\<^sub>"   [(control l)])
-(proof-defshortcut isar-longsup  "\\<^bsup>%p\\<^esup>" [u])
-(proof-defshortcut isar-longsub  "\\<^bsub>%p\\<^esub>" [l])
-(proof-defshortcut isar-idsub    "\\<^isub>" [(control i)])
-(proof-defshortcut isar-raw      "\\<^raw:%p>" [(control r)])
+(proof-defshortcut isar-bold      "\\<^bold>%p" [(control b)])
+(proof-defshortcut isar-local     "\\<^loc>%p" [(control c)])
+(proof-defshortcut isar-super     "\\<^sup>%p" [(control u)])
+(proof-defshortcut isar-sub       "\\<^sub>%p" [(control l)])
+(proof-defshortcut isar-longsuper "\\<^bsup>%p\\<^esup>" [?u])
+(proof-defshortcut isar-longsub   "\\<^bsub>%p\\<^esub>" [?l])
+(proof-defshortcut isar-idsub     "\\<^isub>%p" [(control i)])
+(proof-defshortcut isar-raw       "\\<^raw:%p>" [(control r)])
+(proof-defshortcut isar-antiquote "@{text \"%p\"}" [(control a)])
+(proof-defshortcut isar-ml	  "ML {* %p *}" [(control x)])
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;   Isar shell startup and exit hooks                              ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;borrowed from plastic.el
 (defvar isar-shell-current-line-width nil
   "Current line width of the Isabelle process's pretty printing module.
 Its value will be updated whenever the corresponding screen gets
 selected.")
 
-;borrowed from plastic.el
 (defun isar-shell-adjust-line-width ()
   "Use Isabelle's pretty printing facilities to adjust output line width.
 Checks the width in the `proof-goals-buffer'"
@@ -533,10 +507,9 @@ Checks the width in the `proof-goals-buffer'"
          (save-excursion
            (set-buffer proof-goals-buffer)
            (let ((current-width
-                  ;; Actually, one might sometimes
-                  ;; want to get the width of the proof-response-buffer
-                  ;; instead. Never mind.
-                  (max 20 (window-width (get-buffer-window proof-goals-buffer)))))
+                  ;; Actually, one might want the width of the
+                  ;; proof-response-buffer instead. Never mind.
+                  (max 20 (window-width (get-buffer-window proof-goals-buffer t)))))
 
              (if (equal current-width isar-shell-current-line-width) ()
                (setq isar-shell-current-line-width current-width)
@@ -544,20 +517,12 @@ Checks the width in the `proof-goals-buffer'"
                (setq ans (format "pretty_setmargin %d;" (- current-width 4)))))))
     ans))
 
-(defun isar-pre-shell-start ()
-  (setq proof-prog-name         (isabelle-command-line))
-  (setq proof-mode-for-shell    'isar-shell-mode)
-  (setq proof-mode-for-goals    'isar-goals-mode)
-  (setq proof-mode-for-response 'isar-response-mode)
-  (setq proof-shell-trace-output-regexp "\375"))
-
-
 ;;
 ;;   Configuring proof output buffer
 ;;
 
 (defun isar-preprocessing ()  ;dynamic scoping of `string'
-  "Insert sync markers - acts on variable STRING by dynamic scoping"
+  "Insert sync markers - acts on variable STRING by dynamic scoping."
   (if (proof-string-match isabelle-verbatim-regexp string)
       (setq string (match-string 1 string))
     (unless (proof-string-match ";[ \t]*\\'" string)
@@ -576,61 +541,61 @@ Checks the width in the `proof-goals-buffer'"
                   ;; (if (proof-string-match "\\.ML$" (buffer-name proof-script-buffer))
                   ;;    (format "ML_command {* %s *};" string)
                   ;;    string)
-                  string
+                  (proof-replace-regexp-in-string "\n" "\\\\<^newline>" string)
                   " \\<^sync>;"))))
 
 (defun isar-mode-config ()
   (isar-mode-config-set-variables)
   (isar-init-syntax-table)
   (setq font-lock-keywords isar-font-lock-keywords-1)
-  (proof-config-done)
+  (setq comment-quote-nested nil) ;; can cope with nested comments
   (set (make-local-variable 'outline-regexp) isar-outline-regexp)
   (set (make-local-variable 'outline-heading-end-regexp) isar-outline-heading-end-regexp)
   (setq blink-matching-paren-dont-ignore-comments t)
-  (add-hook 'proof-pre-shell-start-hook 'isar-pre-shell-start nil t)
-  (add-hook 'proof-shell-insert-hook 'isar-preprocessing))
+  (add-hook 'proof-shell-insert-hook 'isar-preprocessing)
+  (proof-config-done))
 
 (defun isar-shell-mode-config ()
   "Configure Proof General proof shell for Isabelle/Isar."
   (isar-init-output-syntax-table)
-  (setq font-lock-keywords  
-	(append 
+  (setq font-lock-keywords
+	(append
 	 isar-output-font-lock-keywords-1
-	 (if (boundp 'x-symbol-isabelle-font-lock-keywords)
-	     x-symbol-isabelle-font-lock-keywords)))
+	 (if (boundp 'x-symbol-isar-font-lock-keywords)
+	     x-symbol-isar-font-lock-keywords)))
   (isar-shell-mode-config-set-variables)
   (proof-shell-config-done))
 
 (defun isar-response-mode-config ()
   (isar-init-output-syntax-table)
-  (setq font-lock-keywords  
-	(append 
+  (setq font-lock-keywords
+	(append
 	 isar-output-font-lock-keywords-1
-	 (if (proof-ass x-symbol-enable)
-	     x-symbol-isabelle-font-lock-keywords)))
+	 (if isar-x-symbol-enable
+	     x-symbol-isar-font-lock-keywords)))
   (proof-response-config-done))
 
 (defun isar-goals-mode-config ()
-  ;; FIXME: next two broken, of course, as is PBP everywhere except LEGO.
-  (setq pg-goals-change-goal "show %s.")
+  (setq pg-goals-change-goal "prefer %s")
   (setq pg-goals-error-regexp proof-shell-error-regexp)
   (isar-init-output-syntax-table)
-  (setq font-lock-keywords  
-	(append 
+  (setq font-lock-keywords
+	(append
 	 isar-goals-font-lock-keywords
-	 (if (proof-ass x-symbol-enable)
-	     x-symbol-isabelle-font-lock-keywords)))
+	 (if isar-x-symbol-enable
+	     x-symbol-isar-font-lock-keywords)))
   (proof-goals-config-done))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;
-;; Remove isa-mode from auto-mode-alist, 
-;; to allow SML mode to work in preference to isa-mode.
-;;
+(defun isar-goalhyplit-test ()
+  "Value for `pg-topterm-goalhyplit-fn' (see that variable for documentation)."
+  (let ((start (point)))
+    (if (proof-re-search-forward
+	 "\377\\|\^AX" nil t) ; end of literal command (non-standard)
+	(progn
+	  (delete-region (match-beginning 0) (match-end 0))
+	  (cons 'lit (buffer-substring start (match-beginning 0)))))))
 
-(setq auto-mode-alist
-      (delete-if 
-       (lambda (strmod) (memq (cdr strmod) '(isa-mode demoisa-mode)))
-       auto-mode-alist))
 
 (provide 'isar)
+
+;;; isar.el ends here
