@@ -5,7 +5,7 @@
 ;; Maintainer: Paul Callaghan <P.C.Callaghan@durham.ac.uk>
 
 ;;
-;; lego.el,v 8.0 2004/04/17 23:40:00 da Exp
+;; lego.el,v 9.0 2008/01/30 15:22:19 da Exp
 ;;
 
 (require 'proof)
@@ -195,9 +195,10 @@ Given is the first SPAN which needs to be undone."
     ;; FIXME: make this stuff generic.  This should be undo-n-times-cmd
     (concat "Undo " (int-to-string ct) ";")))
 
-(defun lego-goal-command-p (str)
+(defun lego-goal-command-p (span)
   "Decide whether argument is a goal or not"
-  (proof-string-match lego-goal-command-regexp str))
+  (proof-string-match lego-goal-command-regexp
+		      (or (span-property span 'cmd) "")))
 
 (defun lego-find-and-forget (span) 
   (let (str ans)
@@ -285,7 +286,7 @@ Checks the width in the `proof-goals-buffer'"
 		;; Actually, one might sometimes
 		;; want to get the width of the proof-response-buffer
 		;; instead. Never mind. 
-		(window-width (get-buffer-window proof-goals-buffer))))
+		(window-width (get-buffer-window proof-goals-buffer t))))
 	   (if (equal current-width lego-shell-current-line-width) ()
 	     ; else
 	     (setq lego-shell-current-line-width current-width)
@@ -293,13 +294,6 @@ Checks the width in the `proof-goals-buffer'"
 	     (insert (format lego-pretty-set-width (- current-width 1)))
 	     )))))
 
-(defun lego-pre-shell-start ()
-  (setq proof-prog-name lego-prog-name
-	proof-mode-for-shell 'lego-shell-mode
-	proof-mode-for-response 'lego-response-mode
-	proof-mode-for-goals 'lego-goals-mode))
-
-
 (defun lego-mode-config ()
 
   (setq proof-terminal-char ?\;)
@@ -308,19 +302,19 @@ Checks the width in the `proof-goals-buffer'"
 
   (setq proof-assistant-home-page lego-www-home-page)
   
-  (setq proof-mode-for-script 'lego-mode)
-
   (setq proof-showproof-command "Prf;"
 	proof-goal-command "Goal %s;"
 	proof-save-command "Save %s;"
 	proof-context-command "Ctxt;"
 	proof-info-command "Help;")
 
+  (setq proof-prog-name lego-prog-name)
+
   (setq proof-goal-command-p 'lego-goal-command-p
 	proof-completed-proof-behaviour 'closeany ; new in 3.0
 	proof-count-undos-fn 'lego-count-undos
 	proof-find-and-forget-fn 'lego-find-and-forget
-        pg-topterm-goalhyp-fn 'lego-goal-hyp
+        pg-topterm-goalhyplit-fn 'lego-goal-hyp
 	proof-state-preserving-p 'lego-state-preserving-p)
 
   (setq	proof-save-command-regexp lego-save-command-regexp
@@ -366,7 +360,6 @@ Checks the width in the `proof-goals-buffer'"
 
 ;; hooks and callbacks
 
-  (add-hook 'proof-pre-shell-start-hook 'lego-pre-shell-start nil t)
   (add-hook 'proof-shell-insert-hook 'lego-shell-adjust-line-width))
 
 (defun lego-equal-module-filename (module filename)
@@ -402,7 +395,7 @@ We assume that module identifiers coincide with file names."
         pg-subterm-start-char ?\372
         pg-subterm-sep-char ?\373
         pg-subterm-end-char ?\374
-        pg-topterm-char ?\375
+        pg-topterm-regexp "\375"
 	proof-shell-eager-annotation-start "\376"
 	proof-shell-eager-annotation-start-length 1
 	proof-shell-eager-annotation-end "\377"
@@ -418,6 +411,9 @@ We assume that module identifiers coincide with file names."
 	proof-shell-process-output-system-specific lego-shell-process-output
         lego-shell-current-line-width nil
 
+	;; LEGO uses Unicode escape prefix: liable to create problems
+	proof-shell-unicode nil 
+
 	proof-shell-process-file
 	(cons "Creating mark \"\\(.*\\)\" \\[\\(.*\\)\\]"  
 	  (lambda (str) (let ((match (match-string 2 str)))
