@@ -1,12 +1,13 @@
-;; Exp 2004/04/17 23:40:00 8.0 
+;; Exp 2008/01/30 15:22:21 9.0 
 
 (require 'proof)			; load generic parts
 
-;; Adjust toolbar entries.  (Must be done
-;; before proof-toolbar is loaded).
+;; Adjust toolbar entries.  (Must be done before proof-toolbar is
+;; loaded).
 
-(if proof-running-on-XEmacs (setq phox-toolbar-entries
-      (remassoc 'context phox-toolbar-entries)))
+(eval-after-load "pg-custom"
+  '(setq phox-toolbar-entries
+    (remassoc 'context phox-toolbar-entries)))
 
 
 ;; ======== User settings for PhoX ========
@@ -32,11 +33,12 @@
   :type 'file
   :group 'phox)
 
-(defcustom phox-sym-lock t
-  "*Whether to use sym-lock or not."
+(defcustom phox-sym-lock-enabled t
+  "*Whether to use x-symbol or not."
   :type 'boolean
   :group 'phox)
 
+
 (defcustom phox-web-page
   "http://www.lama.univ-savoie.fr/~RAFFALLI/phox.html"
   "URL of web page for PhoX."
@@ -78,6 +80,7 @@
 (require 'phox-extraction)
 (require 'phox-tags)
 (require 'phox-outline)
+(require 'phox-pbrpm)
 
 ;; default for tags [da: moved here to fix compilation 15/02/03]
 
@@ -94,11 +97,8 @@
     phox-tags-menu
     (cons
      phox-extraction-menu
-;; not useful ?
-;    '(["Delete symbol around cursor" phox-delete-symbol-around-point t]
-;    ["Delete symbol" phox-delete-symbol t])
       nil)))
-  )
+)
 
 ;;
 ;; ======== Configuration of generic modes ========
@@ -106,6 +106,10 @@
 
 (defun phox-config ()
   "Configure Proof General scripting for PhoX."
+  (if phox-sym-lock-enabled
+      (progn
+	(phox-sym-lock-start)
+	(setq phox-x-symbol-enable nil)))
   (setq
    proof-terminal-char		?\.	; ends every command
    proof-script-command-end-regexp "[.]\\([ \t\n\r]\\)"
@@ -126,7 +130,7 @@
     phox-strict-comments-regexp
     phox-ident-regexp)
    proof-save-with-hole-result     8
-   proof-ignore-for-undo-count     "\\`\\(constraints\\|flag\\|goals\\|pri\\(nt\\(_sort\\)?\\|ority\\)\\|eshow\\|search\\|depend\\)"
+   proof-ignore-for-undo-count     "\\`\\(constraints\\|flag\\|goals\\|pri\\(nt\\(_sort\\)?\\|ority\\)\\|eshow\\|search\\|depend\\|menu_\\|is_\\)"
    proof-non-undoables-regexp      "\\`\\(undo\\|abort\\)"
    proof-shell-error-regexp        "^\\([^ \n\t\r]* \\)?\\(\\(e\\|E\\)rror\\)\\|\\(\\(f\\|F\\)ailure\\)"
    proof-goal-command              "goal %s."
@@ -153,27 +157,12 @@
    proof-shell-prompt-pattern      "\\(>phox> \\)\\|\\(%phox% \\)"
    proof-shell-annotated-prompt-regexp  "\\(>phox> \\)\\|\\(%phox% \\)"
    proof-shell-interrupt-regexp    "Interrupt"
-   proof-shell-start-goals-regexp  "^\\(Here \\(are\\|is\\) the goal\\)\\|\\([0-9]* goals? created\\)"
+   proof-shell-start-goals-regexp  "^\\(Here \\(are\\|is\\) the goal\\)\\|\\([0-9]+ new goals?\\)\\|\\([0-9]+ goals? possibly instanced\\)"
    proof-shell-end-goals-regexp  "^End of goals."
    proof-shell-quit-cmd            "quit."
    proof-shell-restart-cmd         "restart."
    proof-shell-proof-completed-regexp   "^.*^proved"
-   ;; pg-subterm-first-special-char ?\371
-   ;; proof-shell-wakeup-char ?\371
-   ;; pg-subterm-start-char ?\371
-   ;; pg-subterm-sep-char ?\372
-   ;; pg-topterm-char ?\373
-   ;; pg-subterm-end-char ?\374
-   ;; proof-shell-eager-annotation-start "\376"
-   ;; proof-shell-eager-annotation-start-length 1
-   ;; proof-shell-eager-annotation-end "\377"
-;    proof-shell-annotated-prompt-regexp "Lego> \371"
-;    proof-shell-result-start "\372 Pbp result \373"
-;    proof-shell-result-end "\372 End Pbp result \373"
-;    proof-shell-start-goals-regexp "\372 Start of Goals \373"
-;    proof-shell-end-goals-regexp "\372 End of Goals \373"
-;    proof-shell-pre-sync-init-cmd "Configure AnnotateOn;"
-))
+   ))
 
 
 ;;
@@ -189,7 +178,6 @@
 (define-derived-mode phox-mode proof-mode
     "PhoX script" nil
     (phox-config)
-    (phox-sym-lock-start)
     (proof-config-done)
     (phox-setup-outline)
     (define-key phox-mode-map [(control j)] 
@@ -198,7 +186,13 @@
     ;; it is nice to do : xmodmap -e "keysym KP_Enter = Linefeed"
 
     (define-key phox-mode-map [(control c) (meta d)] 
-      'phox-delete-symbol-on-cursor)  
+      'phox-delete-symbol-on-cursor)
+    ;(if phox-x-symbol-enable    
+;	(progn
+	;  (setq x-symbol-language 'phox)
+	 ; (x-symbol-mode t))) ; just to be sure
+;    (font-lock-mode t) ; just to be sure (not always activated on OSX ??)
+
 )
 
 (define-derived-mode phox-shell-mode proof-shell-mode
@@ -212,7 +206,7 @@
    font-lock-keywords (append phox-font-lock-keywords proof-xsym-font-lock-keywords)
    proof-output-fontify-enable     t)
   (phox-sym-lock-start)
-  (if (and (featurep 'phox-sym-lock) phox-sym-lock)
+  (if (and (featurep 'phox-sym-lock) phox-sym-lock-enabled)
       (add-hook 'proof-shell-handle-delayed-output-hook 
 		'phox-sym-lock-font-lock-hook)
     )
@@ -221,35 +215,20 @@
 (define-derived-mode phox-goals-mode proof-goals-mode
   "PhoX goals" nil
   (setq 
-   font-lock-keywords  (append phox-font-lock-keywords proof-xsym-font-lock-keywords)
+   font-lock-keywords (append phox-font-lock-keywords proof-xsym-font-lock-keywords)
    proof-output-fontify-enable     t)
   (phox-sym-lock-start)
-  (if (and (featurep 'phox-sym-lock) phox-sym-lock)
+  (if (and (featurep 'phox-sym-lock) phox-sym-lock-enabled)
       (add-hook 'pg-before-fontify-output-hook 
-		'phox-sym-lock-font-lock-hook))
+		'phox-sym-lock-font-lock-hook)
+    )
   (proof-goals-config-done))
 
-;; The response buffer and goals buffer modes defined above are
-;; trivial.  In fact, we don't need to define them at all -- they
-;; would simply default to "proof-response-mode" and "pg-goals-mode".
 
 ;; A more sophisticated instantiation might set font-lock-keywords to
 ;; add highlighting, or some of the proof by pointing markup
 ;; configuration for the goals buffer.
 
-;; The final piece of magic here is a hook which configures settings
-;; to get the proof shell running.  Proof General needs to know the
-;; name of the program to run, and the modes for the shell, response,
-;; and goals buffers.
-
-(add-hook 'proof-pre-shell-start-hook 'phox-pre-shell-start)
-
-(defun phox-pre-shell-start ()
-  (setq proof-prog-name		phox-prog-name)
-  (setq proof-mode-for-shell    'phox-shell-mode)
-  (setq proof-mode-for-response 'phox-response-mode)
-  (setq proof-mode-for-goals	'phox-goals-mode))
-
 ; completions
 ; dans completions.el
 ;(setq completion-min-length 6)
@@ -262,12 +241,18 @@
   (append phox-top-keywords phox-proof-keywords)
 )
 
+;;;
+;;; X-Symbol
+;;;
+
+(defpgdefault x-symbol-language 'phox)
+
 (eval-after-load "x-symbol-phox"
  ;; Add x-symbol tokens to phox-completion-table and rebuild
  ;; internal completion table if completion is already active
 '(progn
    (defpgdefault completion-table
-     (append (proof-ass completion-table)
+     (append phox-completion-table
 	     (mapcar (lambda (xsym) (nth 2 xsym))
 		     x-symbol-phox-table)))
    (setq proof-xsym-font-lock-keywords
