@@ -1,7 +1,7 @@
 ;; x-symbol-phox.el   
 ;;	Token language "PhoX Symbols" for package x-symbol
 ;;
-;;  ID:         x-symbol-phox.el,v 8.0 2004/04/17 23:40:00 da Exp
+;;  ID:         x-symbol-phox.el,v 9.0 2008/01/30 15:22:21 da Exp
 ;;  Author:     C. Raffalli
 ;;              Updates by Markus Wenzel, Christoph Wedler, David Aspinall.
 ;;  Copyright   1998 Technische Universitaet Muenchen
@@ -11,15 +11,10 @@
 ;; NB: Part of Proof General distribution.
 ;;
 
-;; CW: this sexpr can be deleted with X-Symbol 4.4.3
-(eval-when-compile
-  ;; Next lines should allow this file to work standalone 
-  ;; without proof-x-symbol.el.  See comments further below too.
-  (require 'cl)
-  (ignore-errors (require 'x-symbol-vars)))
+(require 'proof-utils)
 
 (defvar x-symbol-phox-required-fonts nil)
-  
+
 ;;;===========================================================================
 ;;;  General language accesses, see `x-symbol-language-access-alist'
 ;;;===========================================================================
@@ -28,8 +23,10 @@
 ;; it would be handy to be able to use this file away from PG.  
 ;; FIXME: In future could fix things so just 
 ;; (require 'proof-x-symbol) would be enough here.
-(defvar x-symbol-phox-name "PhoX Symbol")
-(defvar x-symbol-phox-modeline-name "phox")
+(require 'proof-x-symbol) 
+;(defvar x-symbol-phox-name "phox")
+;(defvar x-symbol-phox-modeline-name "phox")
+
 
 (defcustom x-symbol-phox-header-groups-alist nil
    "*If non-nil, used in isasym specific grid/menu.
@@ -38,8 +35,8 @@ See `x-symbol-header-groups-alist'."
   :group 'x-symbol-input-init
   :type 'x-symbol-headers)
 
-(defcustom x-symbol-phox-electric-ignore nil
-;;  "[:'][A-Za-z]\\|<=\\|\\[\\[\\|\\]\\]\\|~="
+(defcustom x-symbol-phox-electric-ignore 
+  "/\\\\\\|\\\\/"
   "*Additional Phox version of `x-symbol-electric-ignore'."
   :group 'x-symbol-phox
   :group 'x-symbol-input-control
@@ -51,7 +48,6 @@ See `x-symbol-header-groups-alist'."
 (defvar x-symbol-phox-extra-menu-items nil
   "Extra menu entries for language `phox'.")
 
-
 (defvar x-symbol-phox-token-grammar
   ;; CW: for X-Symbol-4.4.3:
   ;; '(x-symbol-make-grammar ...)
@@ -59,9 +55,12 @@ See `x-symbol-header-groups-alist'."
       (x-symbol-make-grammar
        :encode-spec '(((id . "[_'a-zA-Z0-9]") (op . "[]><=\\/~&+-*%!{}:-]")) .
                     ((id . "[_'a-zA-Z0-9]") (op . "[]><=\\/~&+-*%!{}:-]")))
-       :decode-spec nil
+       :decode-spec '(((id . "[_'a-zA-Z0-9]") (op . "[]><=\\/~&+-*%!{}:-]")) .
+                    ((id . "[_'a-zA-Z0-9]") (op . "[]><=\\/~&+-*%!{}:-]")))
        :decode-regexp "\\([_'a-zA-Z0-9]+\\)\\|\\([]><=\\/~&+-*%!{}:-]+\\)"
-       :token-list #'x-symbol-phox-default-token-list)))
+       :token-list #'x-symbol-phox-default-token-list
+       :input-spec '(((id . "[_'a-zA-Z0-9]") (op . "[]><=\\/~&+-*%!{}:-]")) .
+                    ((id . "[_'a-zA-Z0-9]") (op . "[]><=\\/~&+-*%!{}:-]"))))))
 
 (defvar x-symbol-phox-input-token-grammar
   '("\\([_'a-zA-Z0-9]+\\)\\|\\([]><=\\/~&+-*%!{}:-]+\\)"
@@ -74,10 +73,10 @@ See `x-symbol-header-groups-alist'."
    (lambda (x)
      (cons x 
 	   (cond
-	    ;; CW: where are the shapes `id' and `op' used?
+	    ;; the shapes `id' and `op' are used by x-symbol-phox-token-grammar
 	    ((string-match "\\`[A-Za-z_][A-Za-z_0-9]+\\'" x)
 	     'id)
-	    ((string-match "\\`[<>!+-*/|&]+\\'" x) 
+	    ((string-match "\\`[]><=\\/~&+-*%!{}:-]+\\'" x) 
 	     'op))))
    tokens))
 
@@ -98,6 +97,7 @@ See `x-symbol-header-groups-alist'."
 (defvar x-symbol-phox-image-file-truename-alist nil)
 (defvar x-symbol-phox-image-keywords nil)
 
+
 ;;;===========================================================================
 ;;;  Charsym Info
 ;;;===========================================================================
@@ -148,7 +148,7 @@ See `x-symbol-language-access-alist' for details."
     (notequal "!=")
     (element "in")
     (notelement "notin")
-    (propersubset "<<")
+    (propersubset "subset")
     (intersection "inter")
     (union "union")
     (backslash3 "minus")
@@ -166,8 +166,9 @@ See `x-symbol-language-access-alist' for details."
 (defun x-symbol-phox-prepare-table (table)
   "Prepar the table."
     (mapcar (lambda (entry)
-              (list (car entry) nil
-		    (cadr entry)))
+	      (progn
+		(list (car entry) nil
+		      (cadr entry))))
             table))
 
 (defvar x-symbol-phox-table
@@ -176,6 +177,32 @@ See `x-symbol-language-access-alist' for details."
 
 (provide 'x-symbol-phox)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; User-level settings for X-Symbol 
+;;
+;; this is MODE-ON CODING 8BITS UNIQUE SUBSCRIPTS IMAGE
+(defcustom x-symbol-phox-auto-style
+  '(phox-x-symbol-enable	 ; MODE-ON: whether to turn on interactively
+    nil   ;; x-symbol-coding
+    'null ;; x-symbol-8bits	   [NEVER want it; null disables search]
+    nil   ;; x-symbol-unique
+    nil   ;; x-symbol-subscripts
+    nil)  ;; x-symbol-image
+  "Variable used to document a language access.
+See documentation of `x-symbol-auto-style'."
+  :group 'x-symbol-phox
+  :group 'x-symbol-mode
+  :type 'x-symbol-auto-style)
+
+;; CW: this sexpr can be deleted with X-Symbol 4.4.3
+(eval-when-compile
+  ;; Next lines should allow this file to work standalone 
+  ;; without proof-x-symbol.el.  See comments further below too.
+  (require 'cl)
+  (ignore-errors (require 'x-symbol-vars)))
+
+
 ;;;===========================================================================
 ;;;  Internal
 ;;;===========================================================================
