@@ -17,6 +17,7 @@
 
 
 (require 'proof)
+(require 'span)
 (require 'plastic-syntax)
 
 
@@ -36,9 +37,10 @@
   :type 'string
   :group 'plastic)
 
+(eval-and-compile
 (defvar plastic-lit-string 
   ">" 
-  "*Prefix of literate lines. Set to empty string to get non-literate mode")
+  "*Prefix of literate lines. Set to empty string to get non-literate mode"))
 
 (defcustom plastic-help-menu-list
   '(["The PLASTIC Reference Card" (browse-url plastic-www-refcard) t]
@@ -233,9 +235,10 @@ Given is the first SPAN which needs to be undone."
       (setq span (next-span span 'type)))
     (concat plastic-lit-string " &S Undo x" (int-to-string ct) proof-terminal-string)))
 
-(defun plastic-goal-command-p (str)
+(defun plastic-goal-command-p (span)
   "Decide whether argument is a goal or not"			;; NEED CHG.
-  (proof-string-match plastic-goal-command-regexp str))
+  (proof-string-match plastic-goal-command-regexp 
+		      (or (span-property span 'cmd) "")))
 
 (defun plastic-find-and-forget (span) 
 	;; count the number of spans to undo.
@@ -291,14 +294,15 @@ Given is the first SPAN which needs to be undone."
 ;;   Commands specific to plastic                                      ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;; da: FIXME added quoting/eval here because of macros.  Probably better
-;; to turn proof-defshortcut and co into functions.
-`(proof-defshortcut plastic-Intros 
-		   ,(concat plastic-lit-string "Intros ")  ?i)
-`(proof-defshortcut plastic-Refine
-		   ,(concat plastic-lit-string "Refine ")  ?r)
-`(proof-defshortcut plastic-ReturnAll
-		   ,(concat plastic-lit-string "ReturnAll ") ?u)
+(eval-after-load "plastic" ;; da: so that plastic-lit-string can be changed
+  '(progn
+     (eval `(proof-defshortcut plastic-Intros 
+			       ,(concat plastic-lit-string "Intros ")  [(control i)]))
+     (eval `(proof-defshortcut plastic-Refine
+			       ,(concat plastic-lit-string "Refine ")  [(control r)]))
+     (eval `(proof-defshortcut plastic-ReturnAll
+			       ,(concat plastic-lit-string "ReturnAll ") [(control u)]))))
+
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -325,7 +329,7 @@ Given is the first SPAN which needs to be undone."
                ;; Actually, one might sometimes
                ;; want to get the width of the proof-response-buffer
                ;; instead. Never mind.
-               (window-width (get-buffer-window proof-goals-buffer))))
+               (window-width (get-buffer-window proof-goals-buffer t))))
 
 	   (if (equal current-width plastic-shell-current-line-width) ()
 	     ; else
@@ -334,15 +338,6 @@ Given is the first SPAN which needs to be undone."
 	     (insert (format plastic-pretty-set-width (- current-width 1)))
 	     )))))
 
-(defun plastic-pre-shell-start ()
-  (setq proof-prog-name (concat plastic-prog-name "")
-        ;; set cmd-line flag 
-        proof-mode-for-shell    'plastic-shell-mode
-        proof-mode-for-response 'plastic-response-mode
-        proof-mode-for-goals    'plastic-goals-mode)
-
-  (setenv "PROOF_GENERAL" "")		;; signal to plastic, use annotations
-  )
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -350,16 +345,15 @@ Given is the first SPAN which needs to be undone."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (defun plastic-mode-config ()
-  ;; da: this is now a user-option, please set it in your .emacs
-  ;; via customize mechanism.
-  ;; (setq proof-electric-terminator-enable t)	;; force semicolons active.
 
   (setq proof-terminal-char ?\;)
   (setq proof-script-comment-start "(*")			;; these still active
   (setq proof-script-comment-end "*)")
 
+  (setq proof-prog-name (concat plastic-prog-name ""))
+  (setenv "PROOF_GENERAL" "") ;; signal to plastic, use annotations
+
   (setq proof-assistant-home-page plastic-www-home-page)
-  (setq proof-mode-for-script 'plastic-mode)
 
   (setq proof-showproof-command (concat plastic-lit-string " &S PrfCtxt")
 	proof-goal-command      (concat plastic-lit-string " Claim %s;")
@@ -376,7 +370,7 @@ Given is the first SPAN which needs to be undone."
   (setq proof-goal-command-p 'plastic-goal-command-p
 	proof-count-undos-fn 'plastic-count-undos
 	proof-find-and-forget-fn 'plastic-find-and-forget
-        pg-topterm-goalhyp-fn 'plastic-goal-hyp
+        pg-topterm-goalhyplit-fn 'plastic-goal-hyp
 	proof-state-preserving-p 'plastic-state-preserving-p)
 
   (setq	proof-save-command-regexp plastic-save-command-regexp
@@ -399,12 +393,6 @@ Given is the first SPAN which needs to be undone."
 
   (setq font-lock-keywords plastic-font-lock-keywords-1)
 
-;; FIXME da: this is done generically now.  If you want
-;; the zap commas behaviour, set proof-font-lock-zap-commas=t here.
-;;  (and (boundp 'font-lock-always-fontify-immediately)
-;;       (setq font-lock-always-fontify-immediately t))
-
-
   (proof-config-done)
 
 ;; outline
@@ -430,7 +418,6 @@ Given is the first SPAN which needs to be undone."
 
 ;; hooks and callbacks
 
-  (add-hook 'proof-pre-shell-start-hook    'plastic-pre-shell-start nil t)
   (add-hook 'proof-shell-insert-hook       'plastic-shell-adjust-line-width)
   (add-hook 'proof-shell-handle-error-or-interrupt-hook 'plastic-had-error)
   (add-hook 'proof-shell-insert-hook       'plastic-preprocessing)
@@ -490,7 +477,7 @@ We assume that module identifiers coincide with file names."
         pg-subterm-start-char ?\372
         pg-subterm-sep-char ?\373
         pg-subterm-end-char ?\374
-        pg-topterm-char ?\375
+        pg-topterm-regexp "\375"
         proof-shell-eager-annotation-start "\376"
 	;; FIXME da: if p-s-e-a-s is implemented, you should set
 	;; proof-shell-eager-annotation-start-length=1 to
@@ -585,8 +572,8 @@ We assume that module identifiers coincide with file names."
 	   ))
 	(t 
 	 (incf i))))			; else include.
-     (setq string (replace-in-string string "  *" " "))
-     (setq string (replace-in-string string "^ *" ""))
+     (setq string (replace-in-string string "  +" " "))
+     (setq string (replace-in-string string "^ +" ""))
    (if (string-match "^\\s-*$" string)
        (setq string (concat "ECHO comment line" proof-terminal-string))
      string))))
@@ -660,13 +647,14 @@ We assume that module identifiers coincide with file names."
 
 ;; pcc macros etc
 ;; da: I've moved these key defs out of plastic-mode-config
+;; da: PG 3.5.1: fixed for GNU Emacs compatible syntax
 
-(define-key plastic-keymap ?s 'plastic-small-bar)
-(define-key plastic-keymap ?l 'plastic-large-bar)
-(define-key plastic-keymap ?c 'plastic-all-ctxt)
-(define-key plastic-keymap ?v 'plastic-minibuf)
-(define-key plastic-keymap ?o 'plastic-synchro)
-(define-key plastic-keymap ?p 'plastic-show-shell)
+(define-key plastic-keymap [(control s)] 'plastic-small-bar)
+(define-key plastic-keymap [(control l)] 'plastic-large-bar)
+(define-key plastic-keymap [(control c)] 'plastic-all-ctxt)
+(define-key plastic-keymap [(control v)] 'plastic-minibuf)
+(define-key plastic-keymap [(control o)] 'plastic-synchro)
+(define-key plastic-keymap [(control p)] 'plastic-show-shell)
 
 
 ;; original end.
